#include <i86.h>
#include <string.h>

#include "dpmi.h"

#define DPMI_INT        0x31

union REGS  r;
int         Dpmi_Dos_Error;


// Personal memcopy using selectors

// void memory_to_selector (void* mem_src, selector_t sel_dst, udword size);
// void selector_to_memory (selector_t sel_src, void* mem_dst, udword size);

#if defined (__WATCOMC__)

// void selector_to_memory (selector_t sel_src, void* mem_dst, udword size);
void selector_to_memory (udword sel_src, udword mem_dst, udword size);
#pragma aux selector_to_memory = \
"     push  ds       "\
"     mov   ds,ax    "\
"     mov   esi,0    "\
"     mov   eax,ecx  "\
"     shr   ecx,2    "\
"     rep   movsd    "\
"     mov   ecx,eax  "\
"     and   ecx,3    "\
"     rep   movsb    "\
"     pop   ds       "\
parm [eax] [edi] [ecx];

// void memory_to_selector (void* mem_src, selector_t sel_dst, udword size);
void memory_to_selector (udword mem_src, udword sel_dst, udword size);
#pragma aux memory_to_selector = \
"     push  es       "\
"     mov   es,ax    "\
"     mov   edi,0    "\
"     mov   eax,ecx  "\
"     shr   ecx,2    "\
"     rep   movsd    "\
"     mov   ecx,eax  "\
"     and   ecx,3    "\
"     rep   movsb    "\
"     pop   es       "\
parm [esi] [eax] [ecx];



#elif defined (__GCC__) // DJGPP Conventions
   #define selector_to_memory(ssrc,mdst,size)   dosmemget(ssrc,size,mdst)
   #define memory_to_selector(msrc,sdst,size)   dosmemput(msrc,size,sdst)
#else
   #error Compiler specific function not created
#endif


/* INT 0x2F AX=1680 */
void dpmi_yield (void)
{
   r.rword.ax = 0x1680;
   int386(0x2F,&r,&r);
   return;
}

/* DPMI 0.9 AX=0000 */
word dpmi_allocate_ldt_descriptors (word count, word* descr)
{
   r.rword.ax = 0;
   r.rword.cx = count;
   int386(DPMI_INT,&r,&r);
   *descr = r.rword.ax;
   return (word)r.rdword.cflag;
}

/* DPMI 0.9 AX=0001 */
bool dpmi_free_ldt_descriptor (word descriptor)
{
   r.rword.ax = 1;
   r.rword.cx = descriptor;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0002 */
bool dpmi_segment_to_descriptor (word segm)
{
   r.rword.ax = 0x0002;
   r.rword.bx = segm;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0003 */
bool dpmi_get_selector_increment_value (void)
{
   r.rword.ax = 0x0003;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0006 */
bool dpmi_get_segment_base_address (word selector, dword* addr)
{
   r.rword.ax = 0x0006;
   r.rword.bx = selector;
   int386(DPMI_INT,&r,&r);
   *addr = ((dword)(r.rword.cx<<16))+(dword)r.rword.dx;
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0007 */
bool dpmi_set_segment_base_address (word selector, dword addr)
{
   r.rword.ax = 7;
   r.rword.bx = selector;
   r.rword.cx = (word) (addr >> 16);
   r.rword.dx = (word) (addr & 0xFFFF);
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* LSL instruction  */
dword dpmi_get_segment_limit (word selector)
{
   return 0L;
}

/* DPMI 0.9 AX=0008 */
bool dpmi_set_segment_limit(word selector, dword limit)
{
   r.rword.ax = 8;
   r.rword.bx = selector;
   r.rword.cx = (word) (limit >> 16);
   r.rword.dx = (word) (limit & 0xFFFF);
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* LAR instruction  */
word dpmi_get_descriptor_access_rights (word selector)
{
   return 0;
}

/* DPMI 0.9 AX=0009 */
bool dpmi_set_descriptor_access_rights (word selector, word rights)
{
   r.rword.ax = 0x0009;
   r.rword.bx = selector;
   r.rword.cx = rights;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=000a */
bool dpmi_create_alias_descriptor (word selector)
{
   r.rword.ax = 0x000a;
   r.rword.bx = selector;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=000b */
bool dpmi_get_descriptor (word selector, void* buffer)
{
   r.rword.ax = 0x000b;
   r.rword.bx = selector;
   /* ES:DI buffer */
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=000c */
bool dpmi_set_descriptor (word selector, void* buffer)
{
   r.rword.ax = 0x000c;
   r.rword.bx = selector;
   /* ES:DI buffer */
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=000d */
bool dpmi_allocate_specific_ldt_descriptor(word selector)
{
   r.rword.ax = 0x000d;
   r.rword.bx = selector;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}


/******************************************************************************/


/* DPMI 1.0 AX=000e */
bool dpmi_get_multiple_descriptors (word count, void* buffer)
{
   r.rword.ax = 0x000e;
   r.rword.cx = count;
   /* ES:DI buffer */
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=000f */
bool dpmi_set_multiple_descriptors (word count, void* buffer)
{
   r.rword.ax = 0x000f;
   r.rword.cx = count;
   /* ES:DI buffer */
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}


/******************************************************************************/


/* DPMI 0.9 AX=0100 */
bool dpmi_allocate_dos_memory(word paragraphs, word* ret_selector_or_max, word* ret_dos_segment)
{
   r.rword.ax = 0x0100;
   r.rword.bx = paragraphs;
   int386(DPMI_INT,&r,&r);
   *ret_selector_or_max = r.rword.dx;
   *ret_dos_segment = r.rword.ax;
   Dpmi_Dos_Error = r.rword.ax;
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0101 */
bool dpmi_free_dos_memory(word selector)
{
   r.rword.ax = 0x0101;
   r.rword.dx = selector;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0102 */
bool dpmi_resize_dos_memory(word selector, word newpara, word* ret_max)
{
   r.rword.ax = 0x0102;
   r.rword.bx = newpara;
   r.rword.dx = selector;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}


/******************************************************************************/


/* DPMI 0.9 AX=0200 */
bool dpmi_get_real_mode_interrupt_vector (word vector, dpmi_raddr_t *r_addr)
{
   r.rword.ax = 0x0200;
   r.rbyte.bl = (char) vector;
   int386(DPMI_INT,&r,&r);
   r_addr->segment = r.rword.cx;
   r_addr->offset16 = r.rword.dx;
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0201 */
bool dpmi_set_real_mode_interrupt_vector (word vector, dpmi_raddr_t *r_addr)
{
   r.rword.ax = 0x0201;
   r.rbyte.bl = (char) vector;
   r.rword.cx = r_addr->segment;
   r.rword.dx = r_addr->offset16;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0202 */
bool dpmi_get_processor_exception_handler_vector (word vector, dpmi_paddr_t* p_addr)
{
   r.rword.ax = 0x0202;
   r.rbyte.bl = (char) vector;
   int386(DPMI_INT,&r,&r);
   p_addr->selector = r.rword.cx;
   p_addr->offset32 = r.rdword.edx;
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0203 */
bool dpmi_set_processor_exception_handler_vector (word vector, dpmi_paddr_t* p_addr)
{
   r.rword.ax = 0x0203;
   r.rbyte.bl = (char) vector;
   r.rword.cx = p_addr->selector;
   r.rdword.edx = p_addr->offset32;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0204 */
bool dpmi_get_protected_mode_interrupt_vector (word vector, dpmi_paddr_t *p_addr)
{
   r.rword.ax = 0x0204;
   r.rbyte.bl = (char) vector;
   int386(DPMI_INT,&r,&r);
   p_addr->selector = r.rword.cx;
   p_addr->offset32 = r.rdword.edx;
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0205 */
bool dpmi_set_protected_mode_interrupt_vector (word vector, dpmi_paddr_t *p_addr)
{
   r.rword.ax = 0x0205;
   r.rbyte.bl = (char) vector;
   r.rword.cx = p_addr->selector;
   r.rdword.edx = p_addr->offset32;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}


/******************************************************************************/


/* DPMI 1.0 AX=0210 */
bool dpmi_get_extended_exception_handler_vector_pm (word vector, dpmi_paddr_t* p_addr)
{
   r.rword.ax = 0x0210;
   r.rbyte.bl = (char) vector;
   int386(DPMI_INT,&r,&r);
   p_addr->selector = r.rword.cx;
   p_addr->offset32 = r.rdword.edx;
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0211 */
bool dpmi_get_extended_exception_handler_vector_rm(word vector, dpmi_paddr_t* p_addr)
{
   r.rword.ax = 0x0211;
   r.rbyte.bl = (char) vector;
   int386(DPMI_INT,&r,&r);
   p_addr->selector = r.rword.cx;
   p_addr->offset32 = r.rdword.edx;
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0212 */
bool dpmi_set_extended_exception_handler_vector_pm(word vector, dpmi_paddr_t* p_addr)
{
   r.rword.ax = 0x0212;
   r.rbyte.bl = (char) vector;
   r.rword.cx = p_addr->selector;
   r.rdword.edx = p_addr->offset32;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0213 */
bool dpmi_set_extended_exception_handler_vector_rm(word vector, dpmi_paddr_t* p_addr)
{
   r.rword.ax = 0x0213;
   r.rbyte.bl = (char) vector;
   r.rword.cx = p_addr->selector;
   r.rdword.edx = p_addr->offset32;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}


/******************************************************************************/


bool dpmi_int (word vector, dpmi_regs_t* regs)
/* DPMI 0.9 AX=0300 */
{
   union REGS r;
   struct SREGS sr;

   segread(&sr);

   r.rdword.eax = 0x300;
   r.rdword.ebx = vector;    // bl == vector & bh == 0
   r.rdword.ecx = 0;         // cx : nb word stack to copy pm -> rm
   sr.es = sr.ds;            // es:edi dpmi_regs_t struct adress in pm
   r.rdword.edi  = (udword)(regs);

   int386x(DPMI_INT,&r,&r,&sr);

   return (bool)regs->w.flags;
}

/* DPMI 0.9 AX=0300 */
bool dpmi_simulate_real_mode_interrupt (word vector, dpmi_regs_t* regs)
{
return dpmi_int(vector,regs);
}

void dpmi_int_esdi(byte int_number,dpmi_regs_t* regs, void* buffer, udword buffer_size)
{
   dword nb_paragraphs;
    word pm_selector   = 0;
    word rm_segment    = 0;

   nb_paragraphs = ((buffer_size + (udword)0xF) & (udword)0xFFFFFFF0) >> 4;
   // No overflow test dword -> word
   dpmi_allocate_dos_memory((word)nb_paragraphs,&pm_selector,&rm_segment);
   // no error reported

   regs->rword.es    = rm_segment;
   regs->rdword.edi  = 0;

   memory_to_selector((udword)buffer,pm_selector,buffer_size);
   dpmi_int(int_number,regs);
   selector_to_memory(pm_selector,(udword)buffer,buffer_size);

   dpmi_free_dos_memory(pm_selector);
}

/******************************************************************************/
/******************************************************************************/
/*********                                                             ********/
/*********    UNDER HERE NOTHING IS SURE AND TESTED                    ********/
/*********                                                             ********/
/*********                                            WARNING          ********/
/*********                                                             ********/
/******************************************************************************/
/******************************************************************************/

/* DPMI 0.9 AX=0301 */
bool dpmi_simulate_real_mode_procedure_retf (dpmi_regs_t* regs)
{
   r.rword.ax = 0x0301;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0301 */
bool dpmi_simulate_real_mode_procedure_retf_stack (dpmi_regs_t* regs, word stack_bytes_to_copy, const void* stack_bytes)
{
   r.rword.ax = 0x0301;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0302 */
bool dpmi_simulate_real_mode_procedure_iret (dpmi_regs_t* regs)
{
   r.rword.ax = 0x0302;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0303 */
bool dpmi_allocate_real_mode_callback(void (*_handler)(void), dpmi_regs_t* regs, dpmi_raddr_t* ret)
{
   r.rword.ax = 0x0303;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0304 */
bool dpmi_free_real_mode_callback(dpmi_raddr_t* addr)
{
   r.rword.ax = 0x0304;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0305 */
bool dpmi_get_state_save_restore_addr(dpmi_raddr_t* rm, dpmi_paddr_t* pm)
{
   r.rword.ax = 0x0305;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0306 */
bool dpmi_get_raw_mode_switch_addr(dpmi_raddr_t* rm, dpmi_paddr_t* pm)
{
   r.rword.ax = 0x0306;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0400 */
bool dpmi_get_version(dpmi_version_ret_t* ret)
{
   r.rword.ax = 0x0400;
   int386(DPMI_INT,&r,&r);
   ret->minor    =  r.rbyte.al;
   ret->major    =  r.rbyte.ah;
   ret->flags    =  r.rword.bx;
   ret->cpu      =  r.rbyte.cl;
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0401 */
bool dpmi_get_capabilities(word* flags, char* vendor_info)
{
   r.rword.ax = 0x0401;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0500 */
bool dpmi_get_free_memory_information(dpmi_free_mem_info_t* info)
{
   union REGS regs;
   struct SREGS sregs;
   regs.rdword.eax = 0x0500;
   memset( &sregs, 0, sizeof(sregs) );
   sregs.es = FP_SEG(info);
   regs.rdword.edi = FP_OFF(info);
   int386x( DPMI_INT, &regs, &regs, &sregs );

   return (bool)regs.rdword.cflag;
}

/* DPMI 0.9 AX=0501 */
bool dpmi_allocate_memory(dpmi_meminfo_t* info)
{
   r.rword.ax = 0x0501;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0502 */
bool dpmi_free_memory(dword handle)
{
   r.rword.ax = 0x0502;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0503 */
bool dpmi_resize_memory(dpmi_meminfo_t* info)
{
   r.rword.ax = 0x0503;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0504 */
bool dpmi_allocate_linear_memory(dpmi_meminfo_t* info, word commit)
{
   r.rword.ax = 0x0504;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0505 */
bool dpmi_resize_linear_memory(dpmi_meminfo_t* info, word commit)
{
   r.rword.ax = 0x0505;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0506 */
bool dpmi_get_page_attributes(dpmi_meminfo_t* info, word* buffer)
{
   r.rword.ax = 0x0506;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0507 */
bool dpmi_set_page_attributes(dpmi_meminfo_t* info, word* buffer)
{
   r.rword.ax = 0x0507;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0508 */
bool dpmi_map_device_in_memory_block(dpmi_meminfo_t* info, dword physaddr)
{
   r.rword.ax = 0x0508;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0509 */
bool dpmi_map_conventional_memory_in_memory_block(dpmi_meminfo_t* info, dword physaddr)
{
   r.rword.ax = 0x0509;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=050a */
bool dpmi_get_memory_block_size_and_base(dpmi_meminfo_t* info)
{
   r.rword.ax = 0x050a;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=050b */
bool dpmi_get_memory_information(dpmi_memory_info_t* buffer)
{
   r.rword.ax = 0x050b;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0600 */
bool dpmi_lock_linear_region(dpmi_meminfo_t* info)
{
   r.rword.ax = 0x0600;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0601 */
bool dpmi_unlock_linear_region(dpmi_meminfo_t* info)
{
   r.rword.ax = 0x0601;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0602 */
bool dpmi_mark_real_mode_region_as_pageable(dpmi_meminfo_t* info)
{
   r.rword.ax = 0x0602;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0603 */
bool dpmi_relock_real_mode_region(dpmi_meminfo_t* info)
{
   r.rword.ax = 0x0603;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0604 */
bool dpmi_get_page_size(dword* size)
{
   r.rword.ax = 0x0604;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0702 */
bool dpmi_mark_page_as_demand_paging_candidate(dpmi_meminfo_t* info)
{
   r.rword.ax = 0x0702;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0703 */
bool dpmi_discard_page_contents(dpmi_meminfo_t* info)
{
   r.rword.ax = 0x0703;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0800 */
bool dpmi_physical_address_mapping(dpmi_meminfo_t* info)
{
   r.rword.ax = 0x800;
   r.rword.bx = (uword) (info->address >> 16);
   r.rword.cx = (uword) (info->address & 0xFFFF);
   r.rword.si = (uword) (info->size  >>16 );
   r.rword.di = (uword) (info->size   & 0xFFFF);
   int386(DPMI_INT,&r,&r);
   info->handle = ((udword)r.rword.bx << 16) + (udword)r.rword.cx;
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0801 */
bool dpmi_free_physical_address_mapping(dpmi_meminfo_t* info)
{
   r.rword.ax = 0x0801;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* These next four functions return the old state */
/* DPMI 0.9 AH=09   */
bool dpmi_get_and_set_virtual_interrupt_state(word old_state)
{
   r.rbyte.ah = 0x09;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0900 */
bool dpmi_get_and_disable_virtual_interrupt_state(void)
{
   r.rword.ax = 0x0900;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0901 */
bool dpmi_get_and_enable_virtual_interrupt_state(void)
{
   r.rword.ax = 0x0901;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0902 */
bool dpmi_get_virtual_interrupt_state(void)
{
   r.rword.ax = 0x0902;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0a00 */
bool dpmi_get_vendor_specific_api_entry_point(char *_id, dpmi_paddr_t* api)
{
   r.rword.ax = 0x0a00;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0b00 */
bool dpmi_set_debug_watchpoint(dpmi_meminfo_t* info, word _type)
{
   r.rword.ax = 0x0b00;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0b01 */
bool dpmi_clear_debug_watchpoint(unsigned long _handle)
{
   r.rword.ax = 0x0b01;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0b02 */
bool dpmi_get_state_of_debug_watchpoint(unsigned long _handle, word *_status)
{
   r.rword.ax = 0x0b02;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 0.9 AX=0b03 */
bool dpmi_reset_debug_watchpoint(unsigned long _handle)
{
   r.rword.ax = 0x0b03;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0c00 */
bool dpmi_install_resident_service_provider_callback(dpmi_callback_info_t* info)
{
   r.rword.ax = 0x0c00;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0c01 */
bool dpmi_terminate_and_stay_resident(word return_code, word paragraphs_to_keep)
{
   r.rword.ax = 0x0c01;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0d00 */
bool dpmi_allocate_shared_memory(dpmi_shminfo_t* info)
{
   r.rword.ax = 0x0d00;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0d01 */
bool dpmi_free_shared_memory(dword handle)
{
   r.rword.ax = 0x0d01;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0d02 */
bool dpmi_serialize_on_shared_memory(dword handle, word _flags)
{
   r.rword.ax = 0x0d02;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0d03 */
bool dpmi_free_serialization_on_shared_memory(dword handle, word _flags)
{
   r.rword.ax = 0x0d03;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0e00 */
bool dpmi_get_coprocessor_status(void)
{
   r.rword.ax = 0x0e00;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=0e01 */
bool dpmi_set_coprocessor_emulation(word flags)
{
   r.rword.ax = 0x0e01;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}

/* DPMI 1.0 AX=eeff */
bool dpmi_get_PMODE_infos(void)
{
   r.rword.ax = 0xeeff;
   int386(DPMI_INT,&r,&r);
   return (bool)r.rdword.cflag;
}