////////////////////////////////////////////////////////////////////////////////
// ADD CLIPPING 2D HERE IF NOT USING CSCREEN CLASS
////////////////////////////////////////////////////////////////////////////////

#include <math.h>

#include "fill.h"
#include "light.h"
#include "face.h"
#include "video.h"

#define MIN_X  0
#define MAX_X  1

typedef struct shape_s
{
   sword       min_y;
   sword       max_y;
   rgb_c       color[SCREEN_Y][2];
   vector3_c   coord[SCREEN_Y][2];
   vector3_c   normale[SCREEN_Y][2];
   vector_t    x[SCREEN_Y][2];
}
shape_t;

shape_t Shape;


void reset_edge (void)
{
   Shape.min_y = SCREEN_Y;
   Shape.max_y = 0;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void fill_flat (face_c& face)
{
   set_shape_flat(face,*(face.Object->World->Light));
   draw_shape_flat();
}

void set_shape_flat (face_c& face,light_c& light)
{
   if (face.is_visible())
   {
      Shape.color[0][0] = face.calcul_color(light);
      reset_edge();

      set_edge_flat(*(face.Point1), *(face.Point2));
      set_edge_flat(*(face.Point2), *(face.Point3));
      set_edge_flat(*(face.Point3), *(face.Point1));
   }
   else if (face.TwoSide)
   {
      face.Normale = -face.Normale;

      Shape.color[0][0] = face.calcul_color(light);
      reset_edge();

      set_edge_flat(*(face.Point3), *(face.Point2));
      set_edge_flat(*(face.Point2), *(face.Point1));
      set_edge_flat(*(face.Point1), *(face.Point3));

      // change the way to do that
      face.Normale = -face.Normale;
   }
}

void set_edge_flat (coord_c& point1, coord_c& point2)
{
   sword    min_y, max_y;
   vector_t delta_y;       // Size of the triangle on X axis
   vector_t step_x;        // Inc step on x
   vector_t current_x;     // Pos of the drawing for the Y line
   word  side;             // Min/Max for the array poisition

   if (point1.Point2D[VY] <= point2.Point2D[VY])
   {
      min_y = point1.Point2D[VY]+video_mid_y();
      max_y = point2.Point2D[VY]+video_mid_y();
      delta_y  = max_y-min_y;

      if (delta_y == 0)
         return ;

      side  = MAX_X;
      current_x= point1.Point2D[VX]+video_mid_x();
      step_x   = (point2.Point2D[VX]-point1.Point2D[VX])/delta_y;
   }
   else
   {
      min_y = point2.Point2D[VY]+video_mid_y();
      max_y = point1.Point2D[VY]+video_mid_y();
      delta_y  = max_y-min_y;

      if (delta_y == 0)
         return;

      side  = MIN_X;
      current_x= point2.Point2D[VX]+video_mid_x();
      step_x   = (point1.Point2D[VX]-point2.Point2D[VX])/delta_y;
   }

   if (min_y < Shape.min_y)
      Shape.min_y = min_y;
   if (max_y > Shape.max_y)
      Shape.max_y = max_y;

   while (min_y<max_y)
	{
      Shape.x[min_y][side] = current_x;
      current_x+= step_x;
      min_y++;
   }
}

void draw_shape_flat (void)
{
   word index_x;
   word index_y;

   for(index_y=Shape.min_y; index_y<Shape.max_y; index_y++)
   {
      for(index_x = Shape.x[index_y][MIN_X];
         index_x<Shape.x[index_y][MAX_X];
			index_x++)
      {
         video_plot
         (
            index_x,
            index_y,
            Shape.color[0][0][RED],
            Shape.color[0][0][GREEN],
            Shape.color[0][0][BLUE]
         );
      }
   }
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void fill_gouraud (face_c& face)
{
   set_shape_gouraud(face,*(face.Object->World->Light));
   draw_shape_gouraud();
}

void set_shape_gouraud (face_c& face,light_c& light)
{
   rgb_c c1,c2,c3;

   if (face.is_visible())
   {
      c1 = face.Point1->calcul_color(light);
      c2 = face.Point2->calcul_color(light);
      c3 = face.Point3->calcul_color(light);

      reset_edge();
      set_edge_gouraud(*(face.Point1), *(face.Point2), c1, c2);
      set_edge_gouraud(*(face.Point2), *(face.Point3), c2, c3);
      set_edge_gouraud(*(face.Point3), *(face.Point1), c3, c1);
   }
   else if (face.TwoSide)
   {
      face.Normale = -face.Normale;

      c1 = face.Point1->calcul_color(light);
      c2 = face.Point2->calcul_color(light);
      c3 = face.Point3->calcul_color(light);

      reset_edge();
      set_edge_gouraud(*(face.Point3), *(face.Point2), c3, c2);
      set_edge_gouraud(*(face.Point2), *(face.Point1), c2, c1);
      set_edge_gouraud(*(face.Point1), *(face.Point3), c1, c3);

      // change the way to do that
      face.Normale = -face.Normale;
   }
}

void set_edge_gouraud (coord_c& point1, coord_c& point2, rgb_c& c1, rgb_c& c2)
{
   sword    min_y, max_y;
   vector_t delta_y;       // Size of the triangle on X axis
   vector_t step_x;        // Inc step on x
   vector_t current_x;     // Pos of the drawing for the Y line
   word  side;             // Min/Max for the array poisition
   rgb_c color_base;
   rgb_c color_inc;

   if (point1.Point2D[VY] <= point2.Point2D[VY])
   {
      min_y = point1.Point2D[VY]+video_mid_y();
      max_y = point2.Point2D[VY]+video_mid_y();
      delta_y  = max_y-min_y;

      if (delta_y == 0)
         return;

      side  = MAX_X;
      current_x= point1.Point2D[VX]+video_mid_x();
      step_x   = (point2.Point2D[VX]-point1.Point2D[VX])/delta_y;
      color_base = c1;
      color_inc  = (c2-c1)/delta_y;

   }
   else
   {
      min_y = point2.Point2D[VY]+video_mid_y();
      max_y = point1.Point2D[VY]+video_mid_y();
      delta_y  = max_y-min_y;

      if (delta_y == 0)
         return;

      side  = MIN_X;
      current_x= point2.Point2D[VX]+video_mid_x();
      step_x   = (point1.Point2D[VX]-point2.Point2D[VX])/delta_y;
      color_base = c2;
      color_inc  = (c1-c2)/delta_y;
   }

   if (min_y < Shape.min_y)
      Shape.min_y = min_y;
   if (max_y > Shape.max_y)
      Shape.max_y = max_y;

   while (min_y<max_y)
	{
      Shape.x[min_y][side] = current_x;
      Shape.color[min_y][side] = color_base;
      color_base += color_inc;
      current_x += step_x;
      min_y++;
   }
}

void draw_shape_gouraud (void)
{
   sword    index_x;
   sword    index_y;
   rgb_c    color;
   rgb_c    color_inc;
   vector_t delta_x;

   for(index_y=Shape.min_y; index_y<Shape.max_y; index_y++)
   {
      color = Shape.color[index_y][MIN_X];

      delta_x = Shape.x[index_y][MAX_X]-Shape.x[index_y][MIN_X];

      if (delta_x != 0)
      {
         color_inc = (Shape.color[index_y][MAX_X] - Shape.color[index_y][MIN_X])
         /(delta_x+1);

         for(index_x = Shape.x[index_y][MIN_X];
            index_x<Shape.x[index_y][MAX_X];
            index_x++)
         {
            video_plot
            (
               index_x,
               index_y,
               color[RED],
               color[GREEN],
               color[BLUE]
            );
            color+=color_inc;
         }
      }
   }
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void fill_phong (face_c& face)
{
   set_shape_phong(face,*(face.Object->World->Light));
   draw_shape_phong(face,*(face.Object->World->Light));
}

void set_shape_phong (face_c& face, light_c& light)
{
   if (face.is_visible())
   {
      reset_edge();
      set_edge_phong(*(face.Point1), *(face.Point2));
      set_edge_phong(*(face.Point2), *(face.Point3));
      set_edge_phong(*(face.Point3), *(face.Point1));
   }
   else if(face.TwoSide)
   {
      face.Normale = -face.Normale;

      reset_edge();
      set_edge_phong(*(face.Point3), *(face.Point2));
      set_edge_phong(*(face.Point2), *(face.Point1));
      set_edge_phong(*(face.Point1), *(face.Point3));

      // change the way to do that
      face.Normale = -face.Normale;
   }
}

void set_edge_phong (coord_c& point1, coord_c& point2)
{

   sword       min_y, max_y;
   vector_t    delta_y;       // Size of the triangle on X axis
   vector_t    step_x;        // Inc step on x
   vector_t    current_x;     // Pos of the drawing for the Y line
   word        side;          // Min/Max for the array poisition

   vector3_c   phong_coord;
   vector3_c   phong_coord_inc;
   vector3_c   phong_normale;
   vector3_c   phong_normale_inc;

   if (point1.Point2D[VY] <= point2.Point2D[VY])
   {
      min_y = point1.Point2D[VY]+video_mid_y();
      max_y = point2.Point2D[VY]+video_mid_y();
      delta_y  = max_y-min_y;

      if (delta_y == 0)
         return ;

      side  = MAX_X;
      current_x= point1.Point2D[VX]+video_mid_x();
      step_x   = (point2.Point2D[VX]-point1.Point2D[VX])/delta_y;

      phong_coord = point1.Point3D;
      phong_normale = point1.Normale;
      phong_coord_inc = (point2.Point3D-point1.Point3D)/delta_y;
      phong_normale_inc = (point2.Normale-point1.Normale)/delta_y;
   }
   else
   {
      min_y = point2.Point2D[VY]+video_mid_y();
      max_y = point1.Point2D[VY]+video_mid_y();
      delta_y  = max_y-min_y;

      if (delta_y == 0)
         return;

      side  = MIN_X;
      current_x= point2.Point2D[VX]+video_mid_x();
      step_x   = (point1.Point2D[VX]-point2.Point2D[VX])/delta_y;

      phong_coord     = point2.Point3D;
      phong_coord_inc = (point1.Point3D-point2.Point3D)/delta_y;
      phong_normale      = point2.Normale;
      phong_normale_inc  = (point1.Normale-point2.Normale)/delta_y;
   }

   if (min_y < Shape.min_y)
      Shape.min_y = min_y;
   if (max_y > Shape.max_y)
      Shape.max_y = max_y;

   while (min_y<max_y)
	{
      Shape.x[min_y][side] = current_x;
      Shape.coord[min_y][side] = phong_coord;
      Shape.normale[min_y][side] = phong_normale;
      phong_coord += phong_coord_inc;
      phong_normale += phong_normale_inc;
      current_x+= step_x;
      min_y++;
   }
}

void draw_shape_phong (face_c& face,light_c& light)
{
   sword index_x;
   sword index_y;
   rgb_c color;
   intensity_c intensity;
   vector_t    step_x;

   coord_c     phong_point;
   vector3_c   phong_coord_inc;
   vector3_c   phong_normale_inc;


   for(index_y=Shape.min_y; index_y<Shape.max_y; index_y++)
   {
      step_x = (Shape.x[index_y][MAX_X]-Shape.x[index_y][MIN_X]);

      if (step_x != 0)
      {
         phong_point.Point3D  = Shape.coord[index_y][MIN_X];
         phong_point.Normale  = Shape.normale[index_y][MIN_X];
         phong_point.Material = *(face.Material);

         phong_coord_inc    = (Shape.coord[index_y][MAX_X]-Shape.coord[index_y][MIN_X])/step_x;
         phong_normale_inc  = (Shape.normale[index_y][MAX_X]-Shape.normale[index_y][MIN_X])/step_x;

         for(index_x = Shape.x[index_y][MIN_X];
            index_x<Shape.x[index_y][MAX_X];
            index_x++)
         {
            color = phong_point.calcul_color(light);
            video_plot
            (
               index_x,
               index_y,
               color[RED],
               color[GREEN],
               color[BLUE]
            );

            phong_point.Point3D += phong_coord_inc;
            phong_point.Normale += phong_normale_inc;
         }
      }
   }
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////