/*
 * INCLUDE FILE
 *
 * Version 0.1
 * Copyright 1996 Thomas MANGIN
 */


#ifndef BASE_H
#define BASE_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iostream.h>

#ifdef EOS_CODE
#  include "eos.h"
#endif

#if defined (__WATCOMC__)
#  pragma inline_depth     (255);
#  pragma inline_recursion (off);
#endif

#define FALSE  0
#define TRUE   1
#define OFF    0
#define ON     1
#define NO     0
#define YES    1

#define TEXT_SIZE 1024

/*
typedef enum boolean {
   FALSE = 0,
   TRUE  = -1
} bool_t;
*/

#ifndef __GNUC__
typedef signed short bool;
#endif

typedef unsigned char    byte;      /* Personal preference */
typedef unsigned char   ubyte;
typedef   signed char   sbyte;
typedef unsigned short   word;
typedef unsigned short  uword;
typedef   signed short  sword;
typedef unsigned long   dword;
typedef unsigned long  udword;
typedef   signed long  sdword;

typedef unsigned long   u32;        /* Portable structure reading */
typedef   signed long   s32;        /* Eg : Picture files         */
typedef unsigned short  u16;
typedef   signed short  s16;
typedef unsigned char   u8;
typedef   signed char   s8;
typedef unsigned char   u1;
typedef   signed char   s1;

typedef   char    ascii_t;          /* Ascii value                */
typedef  sbyte    sascii_t;         /* Ascii value                */
typedef  ubyte    uascii_t;         /* Ascii value                */
typedef   byte    scode_t;          /* Scancode value             */
typedef sdword    index_t;          /* Array or file index        */
typedef   word    error_t;          /* Returned error value       */
typedef   void  (*void_function_t)(void);
                                    /* Pointers on function (void)*/

/* A typedef of the EOS is the same than mine */
#if !defined (EOS_CODE) && !defined (__EOS_H__)
typedef   word    selector_t;       /* Cpu selector              */
#endif
typedef   word    segment_t;        /* Cpu segment               */
typedef  dword    offset_pm_t;      /* Cpu protected mode offset */
typedef   word    offset_rm_t;      /* Cpu real      mode offset */
typedef  dword    register_t;       /* Cpu register              */
typedef  dword    flag_t;           /* Cpu pmode flag            */
typedef  dword    stack_t;          /* Cpu stack value           */
typedef  byte*    adress_t;         /* Memory adress             */

enum {VX,VY,VZ,VW};     // axes


#ifdef _PUSHPOP_SUPPORTED
#pragma pack(__push,1);
#else
#pragma pack(1)
#endif

/* Int8086 functions register use */

# define rbyte    h				/* COMMUN MUST BE ON TOP */
# define	rword		w
# define	rdword	x


#if defined (__WATCOMC__)     /* WATCOM  C++ */

# if !defined (__386__)
#  define int386  int86
#  define int386x int86x
# endif

#else if defined (__TURBOC__) /* BORLAND C++ */

# define int386  int86
# define int386x int86x

# undef   rword
# define  rword	x
# undef   rdword

#endif


typedef union
{
  struct
  {
      dword edi;
      dword esi;
      dword ebp;
      dword res;
      dword ebx;
      dword edx;
      dword ecx;
      dword eax;
  } x;

  struct
  {
     word di,  di_hi;
     word si,  si_hi;
     word bp,  bp_hi;
     word res, res_hi;
     word bx,  bx_hi;
     word dx,  dx_hi;
     word cx,  cx_hi;
     word ax,  ax_hi;

     word flags;
     word es;
     word ds;
     word fs;
     word gs;
     word ip;
     word cs;
     word sp;
     word ss;
  } w;

  struct
  {
      byte edi[4];
      byte esi[4];
      byte ebp[4];
      byte res[4];
      byte bl, bh, bl_hi, bh_hi;
      byte dl, dh, dl_hi, dh_hi;
      byte cl, ch, cl_hi, ch_hi;
      byte al, ah, al_hi, ah_hi;
  } h;
} dpmi_regs_t;

#ifdef _PUSHPOP_SUPPORTED
#  pragma pack(__pop);
#endif


#if defined (__GNUC__)
#  define C_CALL
#  define INTERRUPT
#  define FAR
#endif
#if defined (__WATCOMC__)
#  define C_CALL     cdecl
#  define INTERRUPT  __interrupt __far
#  define FAR        __far
#endif

#if defined (__WATCOMC__)
#  define  CPPARGS
#else
#  if defined (__cplusplus)
#    define CPPARGS ...
#  else
#    define CPPARGS
#  endif
#endif

#ifndef START_EXTERN_C
#  ifdef __cplusplus
#     define START_EXTERN_C   extern "C"  {
#     define END_EXTERN_C                 }
#  else
#     define START_EXTERN_C
#     define END_EXTERN_C
#  endif
#endif

#if defined (__WATCOMC__)     /* WATCOM  C++ */

//   void nop (void);
//   #pragma aux nop = "nop";

   #define i3  { break_point(); }
   #define I3  { break_point(); }


   #if defined(__EOS_H__)
      long eos_wait_vbl (void);
   #  pragma aux eos_wait_vbl "_wait_vbl" modify [eax] value [eax]
   #else
      void break_point (void);
   #  pragma aux break_point = "int 3";
   #endif

   void cli (void);
   #pragma aux cli="cli";
   void sti (void);
   #pragma aux sti="sti";

   void disable (void);
   #pragma aux disable="cli";
   void enable (void);
   #pragma aux enable="sti";

#else if defined (__TURBOC__) /* BORLAND C++ */

   void break_point (void);   // You will have to write the code in asm inline
   void nop (void);           // "int 3" and "nop"

   #define i3  { break_point(); }
   #define I3  { break_point(); }

   #define cli()     _disable()
   #define sti()     _enable()
   #define enable()  _enable()
   #define disable() _disable()

#endif

#endif /* BASE_H */