
/*
 * Debug Functions
 *
 * Version 0.1
 * Copyright 1997 Thomas MANGIN
 */

#ifndef FUNCTIONS_DEBUG_H
#define FUNCTIONS_DEBUG_H

#include "error.h"

void __debug_trace_cpu (byte red, byte green, byte blue);
void __debug_set_border (char color);
void __debug_save_color (void);
void __debug_restore_color (void);


#if (DEBUG_LEVEL > 0)
#
   // ADD THE LINE AND FILE WHERE IT OCCURRED

   #define debug_class() \
      if(this == NULL)  { break_point(); throw EFatalInstance(); }

   #define debug_error(boolean_test) \
      if(boolean_test)  { break_point(); throw EFatalRange(); }

   #define debug_if(boolean_test,messy) \
      if(boolean_test)  { break_point(); throw EFatalMessy(messy); }

   #define debug_exit(messy) \
      throw EFatalMessy(messy);
#
#else
#
   #define debug_class()                        /* NO CLASS TEST */
   #define debug_error(boolean_test)            /* NO FUNCTION ENTRY TEST */
   #define debug_if(boolean_test,messy)         /* NO FATAL_IF TEST */
   #define debug_exit(messy)                    /* NO FATAL MESG */
#
#endif

#if (DEBUG_LEVEL > 1)
#
   #define debug_print(messy) \
      Slhan.print(messy,__FILE__,__LINE__)
#
#else
#
   #define debug_print(messy)                   /* NO INFO MESG */
#
#endif


// CPU CONSUMING FUNCTIONS

#if (DEBUG_LEVEL > 2)
#
   #define debug_trace_cpu(r,g,b)   __debug_trace_cpu (r,g,b)
   #define debug_set_border(c)      __debug_set_border (c)
   #define debug_save_color()       __debug_save_color ()
   #define debug_restore_color()    __debug_restore_color ()
#
#else
#
   #define debug_trace_cpu(r,v,b)   /* NO CPU TRACE  */
   #define debug_set_border(c)      /* NO CPU TRACE  */
   #define debug_save_color()       /* NO CPU TRACE  */
   #define debug_restore_color()    /* NO CPU TRACE  */
#
#endif

#endif