
//
// Freely inspired from Realtech dpmi function
// Execom
//


#ifndef FUNCTION_DPMI_H
#define FUNCTION_DPMI_H

#include "base.h"

typedef struct
{
   word segment;
   word offset16;
} dpmi_raddr_t;

typedef struct
{
   word  selector;
  dword  offset32;
} dpmi_paddr_t;

typedef struct
{
   dword handle;       /* 0, 2 */
   dword size;         /* or count */ /* 4, 6 */
   dword address;      /* 8, 10 */
} dpmi_meminfo_t;

typedef struct
{
   byte  major;
   byte  minor;
   word  flags;
   byte  cpu;
   byte  master_pic;
   byte  slave_pic;
} dpmi_version_ret_t;


typedef struct
{
   dword largest_available_free_block_in_bytes;
   dword maximum_unlocked_page_allocation_in_pages;
   dword maximum_locked_page_allocation_in_pages;
   dword linear_address_space_size_in_pages;
   dword total_number_of_unlocked_pages;
   dword total_number_of_free_pages;
   dword total_number_of_physical_pages;
   dword free_linear_address_space_in_pages;
   dword size_of_paging_file_partition_in_pages;
   dword reserved[3];
} dpmi_free_mem_info_t;

typedef struct
{
// dword total_allocated_bytes_of_physical_memory_host;
//	dword total_allocated_bytes_of_virtual_memory_host;
//	dword total_available_bytes_of_virtual_memory_host;
//	dword total_allocated_bytes_of_virtual_memory_vcpu;
// dword total_available_bytes_of_virtual_memory_vcpu;
// dword total_allocated_bytes_of_virtual_memory_client;
// dword total_available_bytes_of_virtual_memory_client;
//	dword total_locked_bytes_of_memory_client;
//	dword max_locked_bytes_of_memory_client;
//	dword highest_linear_address_available_to_client;
//	dword size_in_bytes_of_largest_free_memory_block;
//	dword size_of_minimum_allocation_unit_in_bytes;
//	dword size_of_allocation_alignment_unit_in_bytes;
//	dword reserved[19];
} dpmi_memory_info_t;

typedef struct
{
	dword data16[2];
	dword code16[2];
	 word ip;
	 word reserved;
	dword data32[2];
	dword code32[2];
   dword eip;
} dpmi_callback_info_t;

typedef struct
{
   dword size_requested;
   dword size;
   dword handle;
   dword address;
   dword name_offset;
    word name_selector;
    word reserved1;
   dword reserved2;
} dpmi_shminfo_t;

typedef struct
{
   byte major;
   byte minor;
   byte realmode;
   byte paging;
   byte cpu;
   byte master_pic;
   byte slave_pic;
} dpmi_spec_t;

/* Unless otherwise noted, all functions return 1 on error,
                                                0 on success
   setting Dpmi_Dos_Error when necessary
 */

 START_EXTERN_C

/* INT 0x2F AX=1680 */  void  dpmi_yield(void);

/* DPMI 0.9 AX=0000 */  word  dpmi_allocate_ldt_descriptors          (word count, word* descr);
/* DPMI 0.9 AX=0001 */  bool  dpmi_free_ldt_descriptor               (word descriptor);
/* DPMI 0.9 AX=0002 */  bool  dpmi_segment_to_descriptor             (word segment);
/* DPMI 0.9 AX=0003 */  bool  dpmi_get_selector_increment_value      (void);
/* DPMI 0.9 AX=0006 */  bool  dpmi_get_segment_base_address          (word selector, dword* addr);
/* DPMI 0.9 AX=0007 */  bool  dpmi_set_segment_base_address          (word selector, dword  addr);
/* LSL instruction  */ dword  dpmi_get_segment_limit                 (word selector);
/* DPMI 0.9 AX=0008 */  bool  dpmi_set_segment_limit                 (word selector, dword limit);
/* LAR instruction  */  word  dpmi_get_descriptor_access_rights      (word selector);
/* DPMI 0.9 AX=0009 */  bool  dpmi_set_descriptor_access_rights      (word selector, word rights);
/* DPMI 0.9 AX=000a */  bool  dpmi_create_alias_descriptor           (word selector);
/* dpmi_?et_descriptor are buggy */
/* DPMI 0.9 AX=000b */  bool  dpmi_get_descriptor                    (word selector, void* buffer);
/* DPMI 0.9 AX=000c */  bool  dpmi_set_descriptor                    (word selector, void* buffer);
/* DPMI 0.9 AX=000d */  bool  dpmi_allocate_specific_ldt_descriptor  (word selector);

/* dpmi_?et_multiple_descriptors are buggy */
/* DPMI 1.0 AX=000e */  bool  dpmi_get_multiple_descriptors (word count, void* buffer);
/* DPMI 1.0 AX=000f */  bool  dpmi_set_multiple_descriptors (word count, void* buffer);

/* DPMI 0.9 AX=0100 */  bool  dpmi_allocate_dos_memory   (word paragraphs, word* ret_selector_or_max, word* ret_dos_segment);
/* DPMI 0.9 AX=0101 */  bool  dpmi_free_dos_memory       (word selector);
/* DPMI 0.9 AX=0102 */  bool  dpmi_resize_dos_memory     (word selector, word newpara, word* ret_max);

/* DPMI 0.9 AX=0200 */  bool  dpmi_get_real_mode_interrupt_vector          (word vector, dpmi_raddr_t* r_addr);
/* DPMI 0.9 AX=0201 */  bool  dpmi_set_real_mode_interrupt_vector          (word vector, dpmi_raddr_t* r_addr);
/* DPMI 0.9 AX=0202 */  bool  dpmi_get_processor_exception_handler_vector  (word vector, dpmi_paddr_t* p_addr);
/* DPMI 0.9 AX=0203 */  bool  dpmi_set_processor_exception_handler_vector  (word vector, dpmi_paddr_t* p_addr);
/* DPMI 0.9 AX=0204 */  bool  dpmi_get_protected_mode_interrupt_vector     (word vector, dpmi_paddr_t* p_addr);
/* DPMI 0.9 AX=0205 */  bool  dpmi_set_protected_mode_interrupt_vector     (word vector, dpmi_paddr_t* p_addr);

/* DPMI 1.0 AX=0210 */  bool  dpmi_get_extended_exception_handler_vector_pm(word vector, dpmi_paddr_t* p_addr);
/* DPMI 1.0 AX=0211 */  bool  dpmi_get_extended_exception_handler_vector_rm(word vector, dpmi_paddr_t* p_addr);
/* DPMI 1.0 AX=0212 */  bool  dpmi_set_extended_exception_handler_vector_pm(word vector, dpmi_paddr_t* p_addr);
/* DPMI 1.0 AX=0213 */  bool  dpmi_set_extended_exception_handler_vector_rm(word vector, dpmi_paddr_t* p_addr);

/* DPMI 0.9 AX=0300 */  bool  dpmi_int                            (word vector, dpmi_regs_t* regs);
/* DPMI 0.9 AX=0300 */  bool  dpmi_simulate_real_mode_interrupt   (word vector, dpmi_regs_t* regs);
/* INT USING ES:EDI */  void  dpmi_int_esdi                       (byte int_number,dpmi_regs_t* regs, void* buffer, udword buffer_size);

/* default to zero */   extern short dpmi_int_ss, dpmi_int_sp, dpmi_int_flags;

/* DPMI 0.9 AX=0301 */  bool  dpmi_simulate_rm_proc_retf       (dpmi_regs_t* regs);
/* DPMI 0.9 AX=0301 */  bool  dpmi_simulate_rm_proc_retf_stack (dpmi_regs_t* regs, word stack_bytes_to_copy, const void* stack_bytes);
/* DPMI 0.9 AX=0302 */  bool  dpmi_simulate_rm_proc_iret       (dpmi_regs_t* regs);

/* AFTER HERE NOT REALISED OR CHECKED */

/* DPMI 0.9 AX=0303 */  bool  dpmi_allocate_real_mode_callback (void_function_t handler, dpmi_regs_t* regs, dpmi_raddr_t* ret);
/* DPMI 0.9 AX=0304 */  bool  dpmi_free_real_mode_callback (dpmi_raddr_t* r_addr);

/* DPMI 0.9 AX=0305 */  bool  dpmi_get_state_save_restore_addr (dpmi_raddr_t* rm, dpmi_paddr_t* pm);
/* DPMI 0.9 AX=0306 */  bool  dpmi_get_raw_mode_switch_addr (dpmi_raddr_t* rm, dpmi_paddr_t* pm);
/* DPMI 0.9 AX=0400 */  bool  dpmi_get_version (dpmi_version_ret_t* ret);
/* DPMI 1.0 AX=0401 */  bool  dpmi_get_capabilities(word* flags, char* vendor_info);

/* DPMI 0.9 AX=0500 */  bool  dpmi_get_free_memory_information(dpmi_free_mem_info_t* info);
/* DPMI 0.9 AX=0501 */  bool  dpmi_allocate_memory(dpmi_meminfo_t* info);
/* DPMI 0.9 AX=0502 */  bool  dpmi_free_memory(dword handle);
/* DPMI 0.9 AX=0503 */  bool  dpmi_resize_memory(dpmi_meminfo_t* info);

/* DPMI 1.0 AX=0504 */  bool  dpmi_allocate_linear_memory(dpmi_meminfo_t* info, word commit);
/* DPMI 1.0 AX=0505 */  bool  dpmi_resize_linear_memory(dpmi_meminfo_t* info, word  commit);
/* DPMI 1.0 AX=0506 */  bool  dpmi_get_page_attributes(dpmi_meminfo_t* info, word* buffer);
/* DPMI 1.0 AX=0507 */  bool  dpmi_set_page_attributes(dpmi_meminfo_t* info, word* buffer);
/* DPMI 1.0 AX=0508 */  bool  dpmi_map_device_in_memory_block(dpmi_meminfo_t* info, dword physaddr);
/* DPMI 1.0 AX=0509 */  bool  dpmi_map_conventional_memory_in_memory_block(dpmi_meminfo_t* info, dword physaddr);
/* DPMI 1.0 AX=050a */  bool  dpmi_get_memory_block_size_and_base(dpmi_meminfo_t* info);
/* DPMI 1.0 AX=050b */  bool  dpmi_get_memory_information(dpmi_memory_info_t* buffer);

/* DPMI 0.9 AX=0600 */  bool  dpmi_lock_linear_region(dpmi_meminfo_t* info);
/* DPMI 0.9 AX=0601 */  bool  dpmi_unlock_linear_region(dpmi_meminfo_t* info);
/* DPMI 0.9 AX=0602 */  bool  dpmi_mark_real_mode_region_as_pageable(dpmi_meminfo_t* info);
/* DPMI 0.9 AX=0603 */  bool  dpmi_relock_real_mode_region(dpmi_meminfo_t* info);
/* DPMI 0.9 AX=0604 */  bool  dpmi_get_page_size(dword* size);

/* DPMI 0.9 AX=0702 */  bool  dpmi_mark_page_as_demand_paging_candidate(dpmi_meminfo_t* info);
/* DPMI 0.9 AX=0703 */  bool  dpmi_discard_page_contents(dpmi_meminfo_t* info);

/* DPMI 0.9 AX=0800 */  bool  dpmi_physical_address_mapping(dpmi_meminfo_t* info);
/* DPMI 0.9 AX=0801 */  bool  dpmi_free_physical_address_mapping(dpmi_meminfo_t* info);

/* These next four functions return the old state */
/* DPMI 0.9 AX=0900 */  bool  dpmi_get_and_disable_virtual_interrupt_state(void);
/* DPMI 0.9 AX=0901 */  bool  dpmi_get_and_enable_virtual_interrupt_state(void);
/* DPMI 0.9 AH=09   */  bool  dpmi_get_and_set_virtual_interrupt_state(word old_state);
/* DPMI 0.9 AX=0902 */  bool  dpmi_get_virtual_interrupt_state(void);

/* DPMI 0.9 AX=0a00 */  bool  dpmi_get_vendor_specific_api_entry_point(char* id, dpmi_paddr_t* api);

/* DPMI 0.9 AX=0b00 */  bool  dpmi_set_debug_watchpoint(dpmi_meminfo_t* info, word type);
/* DPMI 0.9 AX=0b01 */  bool  dpmi_clear_debug_watchpoint(dword handle);
/* DPMI 0.9 AX=0b02 */  bool  dpmi_get_state_of_debug_watchpoint(dword dhandle, word* status);
/* DPMI 0.9 AX=0b03 */  bool  dpmi_reset_debug_watchpoint(dword handle);

/* DPMI 1.0 AX=0c00 */  bool  dpmi_install_resident_service_provider_callback(dpmi_callback_info_t* info);
/* DPMI 1.0 AX=0c01 */  bool  dpmi_terminate_and_stay_resident(word return_code, word paragraphs_to_keep);

/* DPMI 1.0 AX=0d00 */  bool  dpmi_allocate_shared_memory(dpmi_shminfo_t* info);
/* DPMI 1.0 AX=0d01 */  bool  dpmi_free_shared_memory(dword handle);
/* DPMI 1.0 AX=0d02 */  bool  dpmi_serialize_on_shared_memory(dword handle, word flags);
/* DPMI 1.0 AX=0d03 */  bool  dpmi_free_serialization_on_shared_memory(dword handle, word flags);

/* DPMI 1.0 AX=0e00 */  bool  dpmi_get_coprocessor_status(void);
/* DPMI 1.0 AX=0e01 */  bool  dpmi_set_coprocessor_emulation(word flags);

END_EXTERN_C

#endif