/*
 * MACRO FILE
 *
 * Version 0.1
 * Copyright 1996 Thomas MANGIN
 */


#ifndef MACRO_H
#define MACRO_H

#include "base.h"

#ifndef Boolean
#define Boolean(X)      (!(!(X)))
#endif

#define Malloc(size)          malloc(size)
#define New(type)             (type *)malloc(sizeof(type))
#define NewArray(type,count)  (type *)malloc(sizeof(type)*(count))
#define NewStr(string)        (char *)strcpy(malloc(strlen(string)+1),string)
#define ZeroNew(count,type)   (type *)calloc(count,sizeof(type))
#define Free(ptr)             free(ptr)

#define  Count(array)         (sizeof(array)/sizeof(array[0]))
#define  Lastof(array)        (Count(array)-1)
#define  Endof(array)         (&(array)[Count(array)])

#define Immed16 (code)  (*(  word*)(code))
#define Immed32 (code)  (*( dword*)(code))
#define Immed32s(code)  (*(sdword*)(code))

#define HiByte(x)       ((byte)((x)>>8)
#define LowByte(x)      ((byte)((x) & 0xFF))

#define HiWord(x)       ((word)((x)>>16))
#define LowWord(x)      ((word)((x) & 0xFFFF))

#define Segment(x)      ((word)((((dword)x) & 0xFFFF0000 ) >>4 ))
#define Offset(x)       ((word)( ((dword)x) & 0x0000FFFF       ))
#define MkPmAddr(x,y)   (((((dword)x) << 4) + (dword)((y) & 0xFFFF)))
#define RmToPmPtr(x)    ((((dword)x>>16)&(0x0000FFFF))<<4)+((dword)x & 0x0000FFFF)

#define MakeAddr(x,y)   (((((dword)x) << 4) + (dword)((y) & 0xFFFF)))

#define Random(a)       (rand()%(a))

#define SetBit(Var,index)   (Var|=(1<<index))
#define ClearBit(Var,index) (Var&=(~(1<<index)))
#define AssignBit(Var,index,value) \
{ \
   Var &= (~(1<<index)); \
   Var |= (Boolean(value)<<index); \
}


//#ifndef __cplusplus

#if ! defined (__GNUC__)
#  define  Min(x,y)  (((x)<(y))?(x):(y))
#  define  Max(x,y)  (((x)>(y))?(x):(y))
#  define  Sign(x)   (((x)>0)?1:-1)
#else
#  define Min(x,y)         \
({                         \
   typeof (x) __x = (x);   \
   typeof (x) __y = (y);   \
   (__x < __y) ? __x : __y;\
})
#  define Max(x,y)         \
({                         \
   typeof (x) __x = (x);   \
   typeof (y) __y = (y);   \
   (__x > __y) ? __x : __y;\
})

#  define Sign(x)          \
({                         \
   typeof (x) __x = (x);   \
   (__x > 0) ? 1 : -1;     \
})

#endif /* __GNUC__ */

/*
#else

template <class T>
inline T Abs(T a)
{
   return(a>=0?a:-a);
}

template <class T>
inline T Max(T a,T b)
{
   return(a>b?a:b);
}

template <class T>
inline T Min(T a,T b)
{
   return(a<b?a:b);
}

template <class T>
inline bool Sign(T a)
{
   return(a>0?1:-1);
}

#endif
*/

template <class T>
inline void Swap(T *a,T *b)
{
   T Swap;
   Swap=*a;
   *a=*b;
   *b=Swap;
}

// float Deg2Rad (float angle)
// float Rad2Deg (float angle)

#define Deg2Rad(angle) (((angle)*PI)/180.0)
#define Rad2Deg(angle) (((angle)*180.0)/PI)


#define Cast(t,f) ((t)(f))
//#define Cast(t,f) (f)

//dword ByteFlipLong (dword NUMBER)
// From Ack3D engine

#define ByteFlipLong (number)       \
{                                   \
   dword    result;                 \
   dword    temp;                   \
   index_t  index;                  \
                                    \
   temp = number;                   \
   result=0;                        \
                                    \
   for (index=0;index<4;index++)    \
   {                                \
      result = result | (T & 0xFF); \
                                    \
      if (index<3)                  \
      {                             \
         result = result << 8;      \
         temp = temp >> 8;          \
      }                             \
   }                                \
   return(result);                  \
}

// word ByteFlipWord (word number)
// From Ack3D engine

#define ByteFlipWord (number)       \
{                                   \
   unsigned short xx1,xx2;          \
   unsigned short result;           \
                                    \
   xx1 = number <<8;                \
   xx2 = number >>8;                \
   result = xx1|xx2;                \
                                    \
   return(result);                  \
}

#endif /* MACRO_H */