#ifndef CLASS_MATERIAL_H
#define CLASS_MATERIAL_H

#include "base3d.h"

#include "rgb.h"

class material_c
{
   private:
      char  Name[20];
      reflection_c DiffuseReflection;
      reflection_c AmbientReflection;
      reflection_c SpecularReflection;

      #define DIFFUSE_REFLECTION_RED        0
      #define DIFFUSE_REFLECTION_GREEN      0
      #define DIFFUSE_REFLECTION_BLUE     255

      #define AMBIENT_REFLECTION_RED       80
      #define AMBIENT_REFLECTION_GREEN     80
      #define AMBIENT_REFLECTION_BLUE     255

      #define SPECULAR_REFLECTION_RED     255
      #define SPECULAR_REFLECTION_GREEN   255
      #define SPECULAR_REFLECTION_BLUE    255

   public:

      material_c (void)
      {
         strcpy(Name,"Default Material");

         DiffuseReflection[RED]   = DIFFUSE_REFLECTION_RED;
         DiffuseReflection[GREEN] = DIFFUSE_REFLECTION_GREEN;
         DiffuseReflection[BLUE]  = DIFFUSE_REFLECTION_BLUE;

         AmbientReflection[RED]   = AMBIENT_REFLECTION_RED;
         AmbientReflection[GREEN] = AMBIENT_REFLECTION_GREEN;
         AmbientReflection[BLUE]  = AMBIENT_REFLECTION_BLUE;

         SpecularReflection[RED]   = SPECULAR_REFLECTION_RED;
         SpecularReflection[GREEN] = SPECULAR_REFLECTION_GREEN;
         SpecularReflection[BLUE]  = SPECULAR_REFLECTION_BLUE;
      }

      char* name (void)
      {
         return Name;
      }

      void name (char* new_name)
      {
         strncpy (Name,new_name,20);
      }

      void erase (void)
      {
         strcpy(Name,"Void Material");

         DiffuseReflection(0,0,0);
         AmbientReflection(0,0,0);
         SpecularReflection(0,0,0);
      }

      reflection_c& diffuse (void)
      {
         return DiffuseReflection;
      }

      reflection_c& ambient (void)
      {
         return AmbientReflection;
      }

      reflection_c& specular (void)
      {
         return SpecularReflection;
      }

      material_c& operator = (const material_c& m)
      {
         DiffuseReflection   = m.DiffuseReflection;
         AmbientReflection   = m.AmbientReflection;
         SpecularReflection  = m.SpecularReflection;
         return *this;
      }

      material_c& operator += (const material_c& m)
      {
         DiffuseReflection   += m.DiffuseReflection;
         AmbientReflection   += m.AmbientReflection;
         SpecularReflection  += m.SpecularReflection;
         return *this;
      }

      material_c& operator /= (const vector_t d)
      {
         DiffuseReflection   /= d;
         AmbientReflection   /= d;
         SpecularReflection  /= d;
         return *this;
      }
};

#endif