//
// Freely inspired from Graphics Gems IV (AP Professional)
//

#ifndef CLASS_MATRIX_3D_H
#define CLASS_MATRIX_3D_H

#include "vector2.h"
#include "vector3.h"

class matrix3_c
{
   protected:
      vector3_c V[3];

   public:
      matrix3_c (void);
      matrix3_c (const vector_t d);
      matrix3_c (const vector3_c& v0, const vector3_c& v1, const vector3_c v2);
      matrix3_c (const matrix3_c& m);

      matrix3_c& operator  = (const matrix3_c& v);
      matrix3_c& operator -= (const matrix3_c& v);
      matrix3_c& operator += (const matrix3_c& v);
      matrix3_c& operator *= (const vector_t d);
      matrix3_c& operator /= (const vector_t d);
      vector3_c& operator [] (const word i);

      matrix3_c  transpose (void);
      matrix3_c  inverse   (void);
      matrix3_c& apply     (vector_function_ptr function);

      friend void swap (matrix3_c& m1, matrix3_c& m2);

      friend matrix3_c operator  - (const matrix3_c& m);
      friend matrix3_c operator  - (const matrix3_c& m1, const matrix3_c& m2);
      friend matrix3_c operator  + (const matrix3_c& m1, const matrix3_c& m2);

      friend matrix3_c operator  * (const matrix3_c& m1, const matrix3_c& m2);
      friend matrix3_c operator  * (const vector_t   d , const matrix3_c& m );
      friend matrix3_c operator  * (const matrix3_c& m , const vector_t   d );
      friend matrix3_c operator  / (const matrix3_c& m , const vector_t   d );

      friend bool operator == (const matrix3_c& m1, const matrix3_c& m2);
      friend bool operator != (const matrix3_c& m1, const matrix3_c& m2);

      friend vector2_c operator  * (const matrix3_c& m, const vector2_c& v);
      friend matrix3_c operator  * (const matrix3_c& m, const vector3_c& v);
};

matrix3_c indentity2D   (void);
matrix3_c translation2D (vector2_c& v);
matrix3_c rotation2D    (vector2_c& center, vector_t angle);
matrix3_c scaling2D     (vector2_c& scale_vector);

#endif