//
// Freely inspired from Graphics Gems IV (AP Professional)
//

#ifndef CLASS_MATRIX_4D_H
#define CLASS_MATRIX_4D_H

#include "vector3.h"
#include "vector4.h"

class matrix4_c
{
   protected:
      vector4_c V[4];

   public:
      matrix4_c (void);
      matrix4_c (const vector_t d);
      matrix4_c (const vector4_c& v0, const vector4_c& v1, const vector4_c v3, const vector4_c v4);
      matrix4_c (const matrix4_c& m);

      matrix4_c& operator  = (const matrix4_c& m);
      matrix4_c& operator -= (const matrix4_c& m);
      matrix4_c& operator += (const matrix4_c& m);
      matrix4_c& operator *= (const vector_t d);
      matrix4_c& operator /= (const vector_t d);
      vector4_c& operator [] (const word i);

      matrix4_c  transpose (void);
      matrix4_c  inverse   (void);
      matrix4_c& apply     (vector_function_ptr function);

      friend void swap (matrix4_c& m1, matrix4_c& m2);

      friend matrix4_c operator  - (const matrix4_c& m);
      friend matrix4_c operator  - (const matrix4_c& m1, const matrix4_c& m2);
      friend matrix4_c operator  + (const matrix4_c& m1, const matrix4_c& m2);

      friend matrix4_c operator  * (const matrix4_c& m1, const matrix4_c& m2);
      friend matrix4_c operator  * (const vector_t   d , const matrix4_c& m );
      friend matrix4_c operator  * (const matrix4_c& m , const vector_t   d );
      friend matrix4_c operator  / (const matrix4_c& m , const vector_t   d );

      friend bool     operator == (const matrix4_c& m1, const matrix4_c& m2);
      friend bool     operator != (const matrix4_c& m1, const matrix4_c& m2);
      friend ostream& operator << (ostream& s, const matrix4_c& M);

      friend vector3_c operator  * (const matrix4_c& m, const vector3_c& v);
      friend vector4_c operator  * (const matrix4_c& m, const vector4_c& v);
};

matrix4_c indentity3D   (void);
matrix4_c translation3D (vector3_c& v);
matrix4_c rotation3D    (vector3_c& axis, vector_t angle);
matrix4_c scaling3D     (vector3_c& scale_vector);
matrix4_c perspective3D (vector_t distance);

#endif