#ifndef CLASS_3DS_H
#define CLASS_3DS_H

/*
   3DS file reader based on the "C" .3DS file format exerciser v1.2.
   Written by Javier Arevalo, AKA Jare/Iguana.
   (0.1 would be a better number as it is somewhere buggy ..)
*/

#include "base3d.h"

class world_c;

class tds_c
{
   public:
      tds_c (world_c* world, char* file_name)
      {
         World = world;

         File = fopen(file_name, "rb");
         if (File == NULL)
         {
            printf("Can't open %s!\n", file_name);
            exit(1);
         }

         fseek(File, 0, SEEK_END);
         FileSize = ftell(File);
         fseek(File, 0, SEEK_SET);

         chunk_reader();
      }

   private:
      static world_c*   World;
      static FILE*      File;
      static sdword     FileSize;
      static sdword     FilePos;
      static word       LastChunk;
      static bool       Verbose;


      typedef struct
      {
          word    id;
          dword   len;
      }
      chunck_header_t;

      typedef struct chunk_name_s
      {
          word id;
          const char *name;
          void (*func)(void);
      }
      chunk_name_t;

      static chunk_name_t ChunkNames[];

      static void chunck_reader (void);
      static void rgb_float_reader (void);
      static void rgb_byte_reader (void);
      static void ambient_reader (void);
      static void diffuse_reader (void);
      static void specular_reader (void);
      static void asciiz_reader (void);
      static void object_block_reader (void);
      static void vertex_list_reader (void);
      static void face_list_reader (void);
      static void object_number_reader(void);

      static sword find_chunk(word id);
      static void chunk_reader(void);
};

#endif