//
// Freely inspired from Graphics Gems IV (AP Professional)
//

#ifndef CLASS_VECTOR_2D_H
#define CLASS_VECTOR_2D_H

#include "base3d.h"

class vector2_c
{
   protected:
      vector_t N[2];

   public:

      vector2_c (void);
      vector2_c (const vector_t d);
      vector2_c (const vector_t x, const vector_t y);
      vector2_c (const vector2_c& v);
      vector2_c (const vector3_c& v);
      vector2_c (const vector3_c& v, word drop_axis);

      vector2_c& operator  = (const vector2_c& v);
      vector2_c& operator -= (const vector2_c& v);
      vector2_c& operator += (const vector2_c& v);
      vector2_c& operator *= (const vector_t   d);
      vector2_c& operator /= (const vector_t   d);
      vector2_c& operator () (const vector_t   x, const vector_t  y);
      vector2_c& operator () (const vector2_c& v);

            vector_t& operator [] (const dword i);
      const vector_t& operator [] (const dword i) const;

      vector_t   length    (void);
      vector_t   length2   (void);
      vector2_c& normalize (void);
      vector2_c& apply     (vector_function_ptr function);
      void       print     (void);

      friend void swap (vector2_c& v1, vector2_c& v2);
      friend vector2_c prod (const vector2_c& v1, const vector2_c& v2);
      friend vector2_c min  (const vector2_c& v1, const vector2_c& v2);
      friend vector2_c max  (const vector2_c& v1, const vector2_c& v2);

      friend vector2_c operator  - (const vector2_c& v);
      friend vector2_c operator  - (const vector2_c& v1, const vector2_c& v2);
      friend vector2_c operator  + (const vector2_c& v1, const vector2_c& v2);

      friend vector_t  operator  * (const vector2_c& v1, const vector2_c& v2);
      friend vector2_c operator  * (const vector2_c& v , const vector_t   d );
      friend vector2_c operator  * (const vector_t   d , const vector2_c& v );
      friend vector2_c operator  * (const vector2_c& v ,       matrix3_c& m );
      friend vector2_c operator  * (const matrix3_c& m , const vector2_c& v );
      friend vector2_c operator  / (const vector2_c& v , const vector_t   d );
      friend vector3_c operator  ^ (const vector2_c& v1, const vector2_c& v2);

      friend bool     operator == (const vector2_c& v1, const vector2_c& v2);
      friend bool     operator != (const vector2_c& v1, const vector2_c& v2);
      friend ostream& operator << (ostream& s, const vector2_c& v);

      friend class vector3_c;
};

#endif