//
// Freely inspired from Graphics Gems IV (AP Professional)
//

#ifndef CLASS_VECTOR_3D_H
#define CLASS_VECTOR_3D_H

#include "base3d.h"

class vector3_c
{
   protected:
      vector_t N[3];

   public:
      vector3_c (void);
      vector3_c (const vector_t d);
      vector3_c (const vector_t x, const vector_t y, const vector_t z);
      vector3_c (const vector3_c& v);
      vector3_c (const vector2_c& v);
      vector3_c (const vector2_c& v, const vector_t d);
      vector3_c (const vector4_c& v);
      vector3_c (const vector4_c& v, word drop_axis);

      vector3_c& operator  = (const vector3_c& v);
      vector3_c& operator -= (const vector3_c& v);
      vector3_c& operator += (const vector3_c& v);
      vector3_c& operator *= (const vector_t   d);
      vector3_c& operator /= (const vector_t   d);
      vector3_c& operator () (const vector_t x, const vector_t y, const vector_t z);
      vector3_c& operator () (const vector3_c& v);
      vector_t&  operator [] (const word i);

		vector_t   length  (void);
		vector_t   length2 (void);
		vector3_c& normalize (void);
		vector3_c& apply (vector_function_ptr function);
		void       print (void);

		friend void swap (vector3_c& v1, vector3_c& v2);
		friend vector3_c prod (const vector3_c& v1, const vector3_c& v2);
		friend vector3_c min  (const vector3_c& v1, const vector3_c& v2);
		friend vector3_c max  (const vector3_c& v1, const vector3_c& v2);

		friend vector3_c operator  - (const vector3_c& v);
		friend vector3_c operator  - (const vector3_c& v1, const vector3_c& v2);
		friend vector3_c operator  + (const vector3_c& v1, const vector3_c& v2);

		friend vector_t  operator  * (const vector3_c& v1, const vector3_c& v2);
		friend vector3_c operator  * (const vector3_c& v , const vector_t   d );
		friend vector3_c operator  * (const vector_t   d , const vector3_c& v );
		friend vector3_c operator  * (const vector3_c& v ,       matrix4_c& m );
		friend vector3_c operator  * (const matrix4_c& m , const vector3_c& v );

		friend vector3_c operator  / (const vector3_c& v , const vector_t   d );
		friend vector3_c operator  ^ (const vector3_c& v1, const vector3_c& v2);

      friend bool       operator == (const vector3_c& v1, const vector3_c& v2);
      friend bool       operator != (const vector3_c& v1, const vector3_c& v2);
      friend ostream&   operator << (ostream& s, const vector3_c& v);

		friend vector_t cosinus (vector3_c& v1, vector3_c& v2);
		friend vector_t   sinus (vector3_c& v1, vector3_c& v2);

      friend class vector2_c;
		friend class vector4_c;
		friend class matrix3_c;
		friend vector2_c operator  * (const matrix3_c& m , const vector2_c& v );
		friend matrix3_c operator  * (const matrix3_c& m1, const matrix3_c& m2);
};

#endif