//
// Freely inspired from Graphics Gems IV (AP Professional)
//

#ifndef CLASS_VECTORD_4D_H
#define CLASS_VECTORD_4D_H

#include "base3d.h"

class vector4_c
{
   protected:
      vector_t N[4];

   public:
      vector4_c (void);
      vector4_c (const vector_t d);
      vector4_c (const vector_t x, const vector_t y, const vector_t z, const vector_t w);
      vector4_c (const vector3_c& v);
      vector4_c (const vector3_c& v, const vector_t d);
      vector4_c (const vector4_c& v);

      vector4_c& operator  = (const vector4_c& v);
      vector4_c& operator -= (const vector4_c& v);
      vector4_c& operator += (const vector4_c& v);
      vector4_c& operator *= (const vector_t d);
      vector4_c& operator /= (const vector_t d);
		vector4_c& operator () (const vector_t x, const vector_t y, const vector_t z, const vector_t w);
      vector4_c& operator () (const vector4_c& v);

            vector_t&  operator [] (const word i);
      const vector_t&  operator [] (const word i) const;

      vector_t length  (void);
      vector_t length2 (void);
      vector4_c& normalize (void);
      vector4_c& homogene  (void);
      vector4_c& apply (vector_function_ptr function);

      friend void swap (vector4_c& v1, vector4_c& v2);
      friend vector4_c prod (const vector4_c& v1, const vector4_c& v2);
      friend vector4_c min  (const vector4_c& v1, const vector4_c& v2);
      friend vector4_c max  (const vector4_c& v1, const vector4_c& v2);

      friend vector4_c operator  - (const vector4_c& v);
      friend vector4_c operator  - (const vector4_c& v1, const vector4_c& v2);
      friend vector4_c operator  + (const vector4_c& v1, const vector4_c& v2);

      friend vector4_c operator  ^ (const vector4_c& v1, const vector4_c& v2);
      friend vector_t  operator  * (const vector4_c& v1, const vector4_c& v2);
      friend vector4_c operator  * (const vector4_c& v , const vector_t   d );
      friend vector4_c operator  * (const vector_t d   , const vector4_c& v );
      friend vector4_c operator  * (const vector4_c& v ,       matrix4_c& m );
      friend vector4_c operator  * (const matrix4_c& m , const vector4_c& v );

      friend vector4_c operator  / (const vector4_c& v , const vector_t   d);
      friend vector4_c operator  ^ (const vector4_c& v1, const vector4_c& v2);

      friend bool     operator == (const vector4_c& v1, const vector4_c& v2);
      friend bool     operator != (const vector4_c& v1, const vector4_c& v2);
      friend ostream& operator << (ostream& s, const vector4_c& v);

      void print (void);

      friend class vector3_c;
      friend class matrix4_c;
      friend vector3_c operator * (const matrix4_c& m , const vector3_c& v );
      friend matrix4_c operator * (const matrix4_c& m1, const matrix4_c& m2);
};

#endif