
//
// Freely inspired from Realtech dpmi function
// Execom
//

#ifndef FUNCTION_VESA_H
#define FUNCTION_VESA_H

#include "base2d.h"

// Video modes definition

#define  VESA_MODE_640x400x256      0x100
#define  VESA_MODE_640x480x256      0x101
#define  VESA_MODE_800x600x16       0x102
#define  VESA_MODE_800x600x256      0x103
#define  VESA_MODE_1024x768x16      0x104
#define  VESA_MODE_1024x768x256     0x105
#define  VESA_MODE_1280x1024x16     0x106
#define  VESA_MODE_1280x1024x256    0x107
#define  VESA_MODE_TEXT_80x60       0x108
#define  VESA_MODE_TEXT_132x25      0x109
#define  VESA_MODE_TEXT_132x43      0x10A
#define  VESA_MODE_TEXT_132x50      0x10B
#define  VESA_MODE_TEXT_132x60      0x10C
#define  VESA_MODE_320x200x32K      0x10D
#define  VESA_MODE_320x200x64K      0x10E
#define  VESA_MODE_320x200x16M      0x10F
#define  VESA_MODE_640x480x32K      0x110
#define  VESA_MODE_640x480x64K      0x111
#define  VESA_MODE_640x480x16M      0x112
#define  VESA_MODE_800x600x32K      0x113
#define  VESA_MODE_800x600x64K      0x114
#define  VESA_MODE_800x600x16M      0x115
#define  VESA_MODE_1024x768x32K     0x116
#define  VESA_MODE_1024x768x64K     0x117
#define  VESA_MODE_1024x768x16M     0x118
#define  VESA_MODE_1280x1024x32K    0x119
#define  VESA_MODE_1280x1024x64K    0x11A
#define  VESA_MODE_1280x1024x16M    0x11B

typedef u16 vmode_t;

typedef struct bgr_s
{
   u8  blue;
   u8  green;
   u8  red;
}
bgr_t;

// Public Variable

extern vmode_t  Vesa_Mode_Video;
extern uword    Vesa_Screen_X;
extern uword    Vesa_Screen_Y;
extern ubyte    Vesa_Bpp;
extern ubyte*   Vesa_Start;
extern ubyte*   Vesa_Video[1024];
extern bool     Vesa_DAC8bit;

// For plotting functions

extern ubyte   Vesa_RM;
extern ubyte   Vesa_RP;
extern ubyte   Vesa_GM;
extern ubyte   Vesa_GP;
extern ubyte   Vesa_BM;
extern ubyte   Vesa_BP;

// Public

bool vesa_is_present (void);
bool vesa_init_system (vmode_t mode);
bool vesa_init_system (word width,word height,byte bbp);
void vesa_original_mode (void);

word vesa_get_mode_by_infos (word width,word height,byte bbp);
bool vesa_get_current_mode (vmode_t* video_mode);
void vesa_clear_screen (void);
void vesa_clear_screen (ubyte color);

bool vesa_set_6bit_palette(void);
bool vesa_set_8bit_palette (void);

void vesa_set_color (index_t index, pixel_t red, pixel_t green, pixel_t blue);
void vesa_set_color (index_t index, rgb_t colors);
void vesa_set_palette (index_t start, index_t number, rgb_t* colors);

void vesa_wait_vbl (void);
void vesa_wait_hbl (void);

bool vesa_get_display_start (coord_t* x, coord_t* y);
bool vesa_set_display_start (coord_t x, coord_t y);


#define rgb_mask(r,g,b) \
( ((r)>>(Vesa_RM)) << Vesa_RP ) | \
( ((g)>>(Vesa_GM)) << Vesa_GP ) | \
( ((b)>>(Vesa_BM)) << Vesa_BP )

#define plot8(x,y,r,g,b)    (( (ubyte*)Vesa_Video[y])[x] = rgb_mask(r,g,b))
#define plot15(x,y,r,g,b)   (( (uword*)Vesa_Video[y])[x] = rgb_mask(r,g,b))
#define plot16(x,y,r,g,b)   (( (uword*)Vesa_Video[y])[x] = rgb_mask(r,g,b))
#define plot32(x,y,r,g,b)   (((udword*)Vesa_Video[y])[x] = rgb_mask(r,g,b))

#define plot24(x,y,r,g,b) \
{ \
   ((bgr_t*)Vesa_Video[y])[x].red     = r; \
   ((bgr_t*)Vesa_Video[y])[x].green   = g; \
   ((bgr_t*)Vesa_Video[y])[x].blue    = b; \
}

#endif