#include "macro.h"

#include "light.h"
#include "object.h"
#include "video.h"

#define ORIGINE_X 0
#define ORIGINE_Y 0
#define ORIGINE_Z 100

#define COLOR_MAX          255

#define LIGHT_COLOR_RED    COLOR_MAX
#define LIGHT_COLOR_GREEN  COLOR_MAX
#define LIGHT_COLOR_BLUE   COLOR_MAX

#define LIGHT_AMBIANT_RED     (COLOR_MAX/5)
#define LIGHT_AMBIANT_GREEN   (COLOR_MAX/5)
#define LIGHT_AMBIANT_BLUE    (COLOR_MAX/5)

#define LIGHT_DIFFUSE_RED     (COLOR_MAX/17)
#define LIGHT_DIFFUSE_GREEN   (COLOR_MAX/17)
#define LIGHT_DIFFUSE_BLUE    (COLOR_MAX/17)

#define LIGHT_DIST_C1   5
#define LIGHT_DIST_C2   5
#define LIGHT_DIST_C3   5

#define PHONG_KS  1
#define PHONG_N   4

#define MAX_WORD_COORD SCREEN_Y


light_c::
light_c (void)
{
   AmbientIntensity[RED]   = LIGHT_AMBIANT_RED  ;
   AmbientIntensity[GREEN] = LIGHT_AMBIANT_GREEN;
   AmbientIntensity[BLUE]  = LIGHT_AMBIANT_BLUE ;

   DiffuseIntensity[RED]   = LIGHT_DIFFUSE_RED  ;
   DiffuseIntensity[GREEN] = LIGHT_DIFFUSE_GREEN;
   DiffuseIntensity[BLUE]  = LIGHT_DIFFUSE_BLUE ;

   Color(LIGHT_COLOR_RED, LIGHT_COLOR_GREEN, LIGHT_COLOR_BLUE);
   Origine(ORIGINE_X,ORIGINE_Y,ORIGINE_Z);
}



rgb_c Color;
rgb_c& phong_color (light_c& light, vector3_c origine, vector3_c normale, material_c& material)
{
   float       distance;
   float       attenuation;
   word        color;

   vector3_c   light_ray;
   vector3_c   view_ray(0,0,1);

   vector3_c   light_and_view;
   vector_t    abs_light_and_view;
   vector3_c   half_ray;


   light_ray = light.Origine.Point3D-origine;
   light_ray.normalize();

   light_and_view = light_ray+view_ray;
   abs_light_and_view = light_and_view.length();
   half_ray = light_and_view / abs_light_and_view;

   distance = (light.Origine.Point3D-origine).length()/MAX_WORD_COORD;

   attenuation = Min
                  ( 1/(
                        LIGHT_DIST_C1+
                        LIGHT_DIST_C2*distance+
                        LIGHT_DIST_C3*distance*distance
                        )
                    ,
                     1
                  );

   for(color=0; color<3; color++)
   {
      Color[color] =
         Min
         (
            light.AmbientIntensity[color]*
            material.ambient()[color]/
            COLOR_MAX+

            attenuation*
            light.DiffuseIntensity[color]*
            material.diffuse()[color]/
            COLOR_MAX*
            (
               Max
               (
                  0
                ,
                  material.specular()[color]*
                  pow((normale*half_ray),PHONG_N)
               )
            )
          ,
            COLOR_MAX
         );
   }

   return Color;
}


   //==============================
   /* Phong Foley 16.15 Page 730 */
   //==============================

/*
rgb_c Color;
rgb_c& phong_color (light_c& light, vector3_c origine, vector3_c normale, material_c& material)
{
   word        color;
   float       f_att;
   float       distance;
   vector3_c   ray;
   vector3_c   view(0,0,1);


   ray = light.Origine.Point3D-origine;
   ray.normalize();

   distance = (light.Origine.Point3D-origine).length()/MAX_WORD_COORD;

   f_att = Min( 1/(
                        LIGHT_DIST_C1+
                        LIGHT_DIST_C2*distance+
                        LIGHT_DIST_C3*distance*distance
                   )
               ,
                1
              );

   for(color=0; color<3; color++)
   {
      Color[color] = (byte)
         Min
         (
            light.AmbientIntensity[color]*
            material.ambient()[color]+

            f_att*
            light.DiffuseIntensity[color]*
            material.diffuse()[color]*
            (
               Max(0,(normale*ray))+

               PHONG_KS*
               material.specular()[color]*
               (normale*ray)*
               pow((2*normale*(normale*ray)-ray)*view,PHONG_N)
            )
          ,
            255
         );
   }

   return Color;
}
*/


/*
rgb_c Color;
rgb_c& phong_color (light_c& light, vector3_c origine, vector3_c normale, material_c& material)
{
   vector3_c   ray;
   vector3_c   view(0,0,1);


   ray = light.Origine.Point3D-origine;
   ray.normalize();

   for(color=0; color<3; color++)
   {
      Color[color] =
         Min
         (
            light.AmbientIntensity[color]*
            material.ambient()[color]/ 255
            +

            light.DiffuseIntensity[color]*
            material.diffuse()[color]*
            (
               Max
               (
                  0
                ,
                  material.specular()[color]*
                  (normale*ray)*
                  pow((2*normale*(normale*ray)-ray)*view,PHONG_N)/255
               )
            )
          ,
            255
         );
   }

   return Color;
}
*/
