#include <new.h>
#include <dos.h>
#include <stdlib.h>

#include "main.h"
#include "error.h"
#include "video.i"

#define CLASS_VERSION_MAJOR   '0'
#define CLASS_VERSION_MINOR   '1'

CMain Slhan;

// Error.h used

bool  IsError;
char* File;
int   Line;
char* Messy;

// C Called function

void new_errror_handler (void)
{
   throw EMemory();
}


// Start

int main (int argc, char** argv)
{
   return Slhan.run(argc,argv);
}

// CMain function

CMain::
CMain ()
{
   FileName   = NULL;
   FileHandle = stdout;

   IsError = FALSE;

   set_new_handler (new_errror_handler);
   exit_code(NULL);
   error_code(NULL);
   error_report(TRUE);
   save_mode();
   set_log(NULL,stderr);
   print_header();
};


CMain::
~CMain()
{
   restore_mode();
   if(ExitCode != NULL && !IsError)
      ExitCode();
}


char* CMain::
version_string ()
{
   static char version[100];

   version[0] = CLASS_VERSION_MAJOR;
   version[1] = '.';
   version[2] = CLASS_VERSION_MINOR;
   version[3] = '\0';

   return version;
}

word  CMain::
vesion_major()
{
   return CLASS_VERSION_MAJOR-'0';
}

word  CMain::
vesion_minor()
{
   return CLASS_VERSION_MINOR-'0';
}


void CMain::
print_header()
{
   word     index;
   word     i_arg;

   int      Argc;
   char**   Argv;
   char*    Param;

   char* SlhanEnv;

   bool  Dos4GQuiet;
   bool  SlhanQuiet;
   bool  Header;
   bool  Info;

   Param = strlwr(getenv("CMDLINE"));

   for (index=0, Argc=1; Param[index] != '\0'; index++)
   {
      if(Param[index] == ' ')
         Argc++;
   }

   Argv = new char*[Argc];
   for (index=0 ; index < Argc; index++)
      Argv[index] = new char[128];

   for (index=0, i_arg=0 ; Param[index] != '\0'; index++)
   {
      if(Param[index] == ' ')
         i_arg++;
      else
         Argv[i_arg][index] = Param[index];
   }

   SlhanEnv = strlwr(getenv("SlhanClass"));
   SlhanQuiet  = (strstr(SlhanEnv,"quiet") != NULL);

   Info     = (Argc>1 && strcmp(strlwr(Argv[1]),"/class=info") == 0);
   Header   = (!SlhanQuiet || Info);

   if(Header)
   {
      if(!Dos4GQuiet)
         printf("\n");
      printf("Slhan's C++ Classes Application                   Version %c.%c\n",CLASS_VERSION_MAJOR,CLASS_VERSION_MINOR);
      printf("Copyrigth (C) 1997                      Slhan / Thomas Mangin\n");
   }

   if(Info)
   {
      printf("\n");
      printf("Compilation date [%s] time [%s]\n",__DATE__,__TIME__);
      printf("Component version not available \n");
      exit(0);
   }
}

void CMain::
set_log (char *file_name, FILE * stream)
{
   if (FileHandle != stdout && FileHandle != stderr)
      fclose (FileHandle);

   if (stream == NULL || stream == stdin)
   {
      FileHandle = stderr;
   }
   else
   {
      if (file_name == NULL)
      {
         FileName   = NULL;
         FileHandle = stream;
      }
      else
      {
         delete FileName;
         FileName = new char[strlen(file_name)+1];
         strcpy(FileName,file_name);
         FileHandle = freopen (file_name, "w+", stream);
      }
   }

   if (ferror (FileHandle) != 0)
   {
      FileHandle = stderr;
   }
}

void CMain::
error_report  (bool state)
{
   ErrorReport = state;
}

void CMain::
error_code (void_function_t code)
{
   ErrorCode = code;
}

void CMain::
exit_code (void_function_t code)
{
   ExitCode = code;
}


void CMain::
error (char* text)
{
   restore_mode ();

   if(ErrorCode != NULL)
      ErrorCode();

   if(ErrorReport)
   {
      #if (DEBUG_LEVEL == 0)
      #
         fprintf(FileHandle, "Application RunTime Error\n");
         fprintf(FileHandle, "%s\n", text);
         fflush(FileHandle);
      #
      #else
      #
         print(text,File,Line);
      #
      #endif
   }
}

void CMain::
print (char* text, char* file, int line)
{
   if (FileName == NULL)
   {
      fprintf (FileHandle, "\n"                        );
      fprintf (FileHandle, "File       : %s\n"  , file );
      fprintf (FileHandle, "Line       : %u\n"  , line );
      fprintf (FileHandle, "Message    : %s\n"  , text );
      fprintf (FileHandle, "\n"                        );
      fflush(FileHandle);
   }
   else
   {
      fprintf (FileHandle, "- File - %-25s " , file );
      fprintf (FileHandle, "- Line - %5u " , line   );
      fprintf (FileHandle, "- Messy - %s " , text   );
      fprintf (FileHandle, "\n"                     );
      fflush(FileHandle);
   }
}

int CMain::
run (int argc, char** argv)
{
   try
   {
      return start(argc,argv);
   }
   catch(ErrExit)
   {
      error(Messy);
      return 0;
   }
   catch(...)
   {
      error("Programmers : You are not catching all your exception");
      return 0;
   }
}