//
// Freely inspired from Graphics Gems IV (AP Professional)
//

#include "debug.h"
#include "macro.h"

#include "rgb.h"

rgb_c::
rgb_c (void)
{
   C.dword_value.red  = 0;
   C.dword_value.green= 0;
   C.dword_value.blue = 0;
}

rgb_c::
rgb_c (const ubyte r, const ubyte g, const ubyte b)
{
   C.dword_value.red  = 0;
   C.dword_value.green= 0;
   C.dword_value.blue = 0;

   C.byte_value.red  = r;
   C.byte_value.green= g;
   C.byte_value.blue = b;
}

rgb_c::
rgb_c (const sdword r, const sdword g, const sdword b, const sdword diff)
{
   C.dword_value.red  = r;
   C.dword_value.green= g;
   C.dword_value.blue = b;
}

rgb_c::
rgb_c (const rgb_c& c)
{
   C.dword_value.red  = c.C.dword_value.red;
   C.dword_value.green= c.C.dword_value.green;
   C.dword_value.blue = c.C.dword_value.blue;
}

rgb_c& rgb_c::
operator = (const rgb_c& c)
{
   C.dword_value.red  = c.C.dword_value.red;
   C.dword_value.green= c.C.dword_value.green;
   C.dword_value.blue = c.C.dword_value.blue;
   return *this;
}

rgb_c& rgb_c::
operator -= (const rgb_c& c)
{
   C.dword_value.red  -= c.C.dword_value.red;
   C.dword_value.green-= c.C.dword_value.green;
   C.dword_value.blue -= c.C.dword_value.blue;
   return *this;
}

rgb_c&
rgb_c:: operator += (const rgb_c& c)
{
   C.dword_value.red  += c.C.dword_value.red;
   C.dword_value.green+= c.C.dword_value.green;
   C.dword_value.blue += c.C.dword_value.blue;
   return *this;
}

rgb_c& rgb_c::
operator *= (const sdword d)
{
   C.dword_value.red  *= d;
   C.dword_value.green*= d;
   C.dword_value.blue *= d;
   return *this;
}

rgb_c& rgb_c::
operator /= (const float d)
{
/*
   float i = 1/d;
   debug_if(d==0, "division by zero");

   C.dword_value.red  = (sdword)((float)C.dword_value.red*i);
   C.dword_value.green= (sdword)((float)C.dword_value.green*i);
   C.dword_value.blue = (sdword)((float)C.dword_value.blue*i);
*/
   debug_if(d==0, "division by zero");

   C.dword_value.red  = C.dword_value.red/d;
   C.dword_value.green= C.dword_value.green/d;
   C.dword_value.blue = C.dword_value.blue/d;

   return *this;
}

rgb_c& rgb_c::
operator /= (const sdword d)
{
   C.dword_value.red  = C.dword_value.red/d;
   C.dword_value.green= C.dword_value.green/d;
   C.dword_value.blue = C.dword_value.blue/d;
   return *this;
}

rgb_c& rgb_c::
operator () (const ubyte r, const ubyte g, const ubyte b)
{
   C.dword_value.red  = 0;
   C.dword_value.green= 0;
   C.dword_value.blue = 0;

   C.byte_value.red  = r;
   C.byte_value.green= g;
   C.byte_value.blue = b;
   return *this;
}

rgb_c& rgb_c::
operator () (const rgb_c& c)
{
   C.dword_value.red  = c.C.dword_value.red;
   C.dword_value.green= c.C.dword_value.green;
   C.dword_value.blue = c.C.dword_value.blue;
	return *this;
}


ubyte& rgb_c::
operator [] (const word i)
{
   switch (i)
   {
      case RED:
         return C.byte_value.red;
      case GREEN:
         return C.byte_value.green;
      case BLUE:
         return C.byte_value.blue;
      default:
         fatal("illegal index access");
   }
   return C.byte_value.red;   // To remove a compilation error
}

ubyte& rgb_c::
red (void)
{
   return C.byte_value.red;
}

ubyte& rgb_c::
green (void)
{
   return C.byte_value.green;
}

ubyte& rgb_c::
blue (void)
{
   return C.byte_value.blue;
}


void rgb_c::
print (void)
{
   printf("(Red %d, Green %d, Blue %d)",C.byte_value.red,C.byte_value.green,C.byte_value.blue);
}

void swap (rgb_c& c1, rgb_c& c2)
{
   rgb_c temp(c1);
   c1 = c2;
   c2 = temp;
}

rgb_c min (const rgb_c& c1,const rgb_c& c2)
{
   return rgb_c(Min(c1.C.dword_value.red,c2.C.dword_value.red),Min(c1.C.dword_value.green,c2.C.dword_value.green),Min(c1.C.dword_value.blue,c2.C.dword_value.blue),0);
}

rgb_c max (const rgb_c& c1,const rgb_c& c2)
{
   return rgb_c(Max(c1.C.dword_value.red,c2.C.dword_value.red),Max(c1.C.dword_value.green,c2.C.dword_value.green),Max(c1.C.dword_value.blue,c2.C.dword_value.blue),0);
}

rgb_c operator - (const rgb_c& c1, const rgb_c& c2)
{
   return rgb_c(c1.C.dword_value.red-c2.C.dword_value.red,c1.C.dword_value.green-c2.C.dword_value.green,c1.C.dword_value.blue-c2.C.dword_value.blue,0);
}

rgb_c operator + (const rgb_c& c1, const rgb_c& c2)
{
   return rgb_c(c1.C.dword_value.red+c2.C.dword_value.red,c1.C.dword_value.green+c2.C.dword_value.green,c1.C.dword_value.blue+c2.C.dword_value.blue,0);
}

rgb_c operator * (const rgb_c& c, const sdword d)
{
   return rgb_c(d*c.C.dword_value.red,d*c.C.dword_value.green,d*c.C.dword_value.blue,0);
}

rgb_c operator * (const sdword d, const rgb_c& c)
{
   return rgb_c(d*c.C.dword_value.red,d*c.C.dword_value.green,d*c.C.dword_value.blue,0);
}

rgb_c operator / (const rgb_c& c, const sdword d)
{
   float i = 1.0/d;
   debug_if(d==0, "division by zero");

   return rgb_c(
                  (sdword)(i*((float)c.C.dword_value.red)),
                  (sdword)(i*((float)c.C.dword_value.green)),
                  (sdword)(i*((float)c.C.dword_value.blue)),
                  0
               );
}

rgb_c operator / (const rgb_c& c, const float d)
{
   float i = 1.0/d;
   debug_if(d==0, "division by zero");

   return rgb_c(
                  (sdword)(i*((float)c.C.dword_value.red)),
                  (sdword)(i*((float)c.C.dword_value.green)),
                  (sdword)(i*((float)c.C.dword_value.blue)),
                  0
               );
}

bool operator == (const rgb_c& c1, const rgb_c& c2)
{
   return (c1.C.byte_value.red==c2.C.byte_value.red) && (c1.C.byte_value.green==c2.C.byte_value.green) && (c1.C.byte_value.blue==c2.C.byte_value.blue);
}

bool operator != (const rgb_c& c1, const rgb_c& c2)
{
   return !(c1==c2);
}

ostream& operator << (ostream& s, const rgb_c& c)
{
    return s << "(" << c.C.byte_value.red << ", " << c.C.byte_value.green << ", " << c.C.byte_value.blue << ")";
}