#include "tds.h"
#include "world.h"

// One day : Verify the 3DS chunk hierarchy

#define CHUNK_HEADER_SIZE 6

world_c*    tds_c::World;
FILE*       tds_c::File;
sdword      tds_c::FileSize;
sdword      tds_c::FilePos;
word        tds_c::LastChunk;
bool        tds_c::Verbose = TRUE;

enum {
   UNKNOW                                 = 0x0000,   // Home made chunk ..
   M3DMAGIC                               = 0x4D4D,
   VERSION                                = 0x0002,
   MLIBMAGIC                              = 0x3DAA,
   CMAGIC                                 = 0xC23D,
   MDATA                                  = 0x3D3D,
      MESH_VERSION                        = 0x3D3E,
      MAT_ENTRY                           = 0xAFFF,
         MAT_NAME                         = 0xA000,
         MAT_AMBIANT                      = 0xA010,
            COLOR_24                      = 0x0011,
            LIN_COLOR_24                  = 0x0012,
         MAT_DIFFUSE                      = 0xA020,
//          COLOR_24
//          LIN_COLOR_24
         MAT_SPECULAR                     = 0xA030,
//          COLOR_24
//          LIN_COLOR_24
         MAT_SHININESS                    = 0xA040,
            INT_PERCENTAGE                = 0x0030,
         MAT_SHIN2PCT                     = 0xA041,
//          INT_PERCENTAGE
         MAT_TRANSPARENCY                 = 0xA050,
//          INT_PERCENTAGE
         MAT_XPFALL                       = 0xA052,
//          INT_PERCENTAGE
         MAT_USE_XPFALL                   = 0xA240,
         MAT_REFBLUR                      = 0xA053,
//          INT_PERCENTAGE
         MAT_SHADING                      = 0xA100,
         MAT_USE_REFBLUR                  = 0xA250,
         MAT_SELF_ILLUM                   = 0xA080,
         MAT_TWO_SIDE                     = 0xA081,
         MAT_DECAL                        = 0xA082,
         MAT_ADDITIVE                     = 0xA083,
         MAT_WIRE                         = 0xA085,
         MAT_FACEMAP                      = 0xA088,
         MAT_PHONGSOFT                    = 0xA08C,
         MAT_WIREABS                      = 0xA08E,
         MAT_WIRESIZE                     = 0xA087,
         MAT_TEXMAP                       = 0xA200,
//          INT_PERCENTAGE
            MAT_MAPNAME                   = 0xA300,
            MAT_MAP_TILING                = 0xA351,
            MAT_MAP_TEXBLUR               = 0xA353,
            MAT_MAP_USCALE                = 0xA354,
            MAT_MAP_VSCALE                = 0xA356,
            MAT_MAP_UOFFSET               = 0xA358,
            MAT_MAP_VOFFSET               = 0xA35A,
            MAT_MAP_ANG                   = 0xA35C,
            MAT_MAP_COL1                  = 0xA360,
            MAT_MAP_COL2                  = 0xA362,
            MAT_MAP_RCOL                  = 0xA364,
            MAT_MAP_GCOL                  = 0xA366,
            MAT_MAP_BCOL                  = 0xA368,
         MAT_SXP_TEXT_DATA                = 0xA320,
         MAT_TEXMASK                      = 0xA33E,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
         MAT_SXP_TEXT_MASKDATA            = 0xA32A,
         MAT_TEX2MAP                      = 0xA33A,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
//          MAT_MAP_COL1
//          MAT_MAP_COL2
//          MAT_MAP_RCOL
//          MAT_MAP_GCOL
//          MAT_MAP_BCOL
         MAT_SXP_TEXT2_DATA               = 0xA321,
         MAT_TEX2MASK                     = 0xA340,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
         MAT_SXP_TEXT2MASK_DATA           = 0xA32C,
         MAT_OPACMAP                      = 0xA210,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
         MAT_SXP_OPAC_DATA                = 0xA322,
         MAT_OPACMASK                     = 0xA342,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
         MAT_SXP_OPACMASK_DATA            = 0xA32E,
         MAT_BUMPMAP                      = 0xA230,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
         MAT_SXP_BUMP_DATA                = 0xA324,
         MAT_BUMPMASK                     = 0xA344,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
         MAT_SXP_BUMPMASK_DATA            = 0xA330,
         MAT_SPECMAP                      = 0xA204,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
//          MAT_MAP_COL1
//          MAT_MAP_COL2
//          MAT_MAP_RCOL
//          MAT_MAP_GCOL
//          MAT_MAP_BCOL
         MAT_SXP_SPEC_DATA                = 0xA325,
         MAT_SPECMASK                     = 0xA348,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
         MAT_SXP_SPECMASK_DATA            = 0xA332,
         MAT_MATSHINMAP                   = 0xA33C,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
         MAT_SXP_SHIN_DATA                = 0xA326,
         MAT_SHINMASK                     = 0xA346,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
         MAT_SXP_SHINMASK_DATA            = 0xA334,
         MAT_SELFIMAP                     = 0xA33D,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
         MAT_SXP_SELFI_DATA               = 0xA328,
         MAT_SELFIMASK                    = 0xA34A,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
         MAT_SXP_SELFIMASK_DATA           = 0xA336,
         MAT_REFLMAP                      = 0xA220,
//          INT_PERCENTAGE
//          MAT_MAPNAME
         MAT_REFLMASK                     = 0xA34C,
//          INT_PERCENTAGE
//          MAT_MAPNAME
//          MAT_MAP_TILING
//          MAT_MAP_TEXBLUR
//          MAT_MAP_USCALE
//          MAT_MAP_VSCALE
//          MAT_MAP_UOFFSET
//          MAT_MAP_VOFFSET
//          MAT_MAP_ANG
         MAT_SXP_REFL_MASKDATA            = 0xA338,
         MAT_ACUBIC                       = 0xA310,
      MASTER_SCALE                        = 0x0100,
      VIEWPORT_LAYOUT                     = 0x7001,
         VIEWPORT_SIZE                    = 0x7020,
         VIEWPORT_DATA                    = 0x7011, // OR
         VIEWPORT_DATA_3                  = 0x7012,
//       VIEWPORT_DATA   OR   VIEWPORT_DATA_3
//       VIEWPORT_DATA   OR   VIEWPORT_DATA_3
//       VIEWPORT_DATA   OR   VIEWPORT_DATA_3
//       VIEWPORT_DATA   OR   VIEWPORT_DATA_3
      LO_SHADOW_BIAS                      = 0x1400,
      HI_SHADOW_BIAS                      = 0x1410,
      SHADOW_MAP_SIZE                     = 0x1420,
      SHADOW_SAMPLES                      = 0x1430,
      SHADOW_RANGE                        = 0x1440,
      SHADOW_FILTER                       = 0x1450,
      RAY_BIAS                            = 0x1460,
      O_CONSTS                            = 0x1500,
      AMBIENT_LIGHT                       = 0x2100,
         COLOR_F                          = 0x0010, // OR
         LIN_COLOR_F                      = 0x0013,
      BIT_MAP                             = 0x1100,
      SOLID_BGND                          = 0x1200,
//       COLOR_F   OR   LIN_COLOR_F
      V_GRADIENT                          = 0x1300,
//       COLOR_F   OR   LIN_COLOR_F
//       COLOR_F   OR   LIN_COLOR_F
//       COLOR_F   OR   LIN_COLOR_F
      USE_BIT_MAP                         = 0x1101,
      USE_SOLID_BGND                      = 0x1201,
      USE_V_GRADIENT                      = 0x1301,
      LAYER_FOG                           = 0x2302,
//       COLOR_F
      FOG                                 = 0x2200,
//       COLOR_F
         FOG_BGND                         = 0x2210,
//    LAYER_FOG
      DISTANCE_CUE                        = 0x2300,
         DCUE_BGND                        = 0x2310,
      USE_FOG                             = 0x2201,
      USE_LAYER_FOG                       = 0x2303,
      USE_DISTANCE_CUE                    = 0x2301,
      DEFAULT_VIEW                        = 0x3000,
         VIEW_TOP                         = 0x3010, // OR
         VIEW_BOTTOM                      = 0x3020, // OR
         VIEW_LEFT                        = 0x3030, // OR
         VIEW_RIGHT                       = 0x3040, // OR
         VIEW_FRONT                       = 0x3050, // OR
         VIEW_BACK                        = 0x3060, // OR
         VIEW_USER                        = 0x3070, // OR
         VIEW_CAMERA                      = 0x3080,
      NAMED_OBJECT                        = 0x4000,
         N_TRI_OBJECT                     = 0x4100,
            POINT_ARRAY                   = 0x4110,
            POINT_FLAG_ARRAY              = 0x4111,
            FACE_ARRAY                    = 0x4120,
               MSH_MAT_GROUP              = 0x4130,
               SMOOTH_GROUP               = 0x4150,
               MSH_BOXMAP                 = 0x4190,
//          MSH_MAT_GROUP                 STRANGE ...
            TEXT_VERTS                    = 0x4140,
            MESH_MATRIX                   = 0x4160,
            MESH_COLOR                    = 0x4165,
            MESH_TEXTURE_INFO             = 0x4170,
            PROC_NAME                     = 0x4181,
            PROC_DATA                     = 0x4182,
         N_DIRECT_LIGHT                   = 0x4600,
//          COLOR_F
            DL_OFF                        = 0x4620,
            DL_OUTER_RANGE                = 0x465A,
            DL_INNER_RANGE                = 0x4659,
            DL_MULTIPLIER                 = 0x465B,
            DL_EXCLUDE                    = 0x4654,
            DL_SPOTLIGHT                  = 0x4610,
               DL_SPOT_ROLL               = 0x4656,
               DL_SHADOWED                = 0x4630,
               DL_LOCAL_SHADOW2           = 0x4641,
               DL_SEE_CONE                = 0x4650,
               DL_SPOT_RECTANGULAR        = 0x4651,
               DL_SPOT_ASPECT             = 0x4657,
               DL_SPOT_PROJECTOR          = 0x4653,
               DL_SPOT_OVERSHOOT          = 0x4652,
               DL_RAY_BIAS                = 0x4658,
               DL_RAYSHAD                 = 0x4627,
            DL_ATTENUATE                  = 0x4625,
         N_CAMERA                         = 0x4700,
            CAMERA_SEE_CONE               = 0x4710,
            CAM_RANGES                    = 0x4720,
         OBJ_HIDDEN                       = 0x4010,
         OBJ_VIS_LOFTER                   = 0x4011,
         OBJ_DOESNT_CAST                  = 0x4012,
         OBJ_MATTE                        = 0x4013,
         OBJ_DONT_RCV_SHADOW              = 0x4017,
         OBJ_FAST                         = 0x4014,
         OBJ_PROCEDURAL                   = 0x4015,
         OBJ_FROZEN                       = 0x1016,
   KFDATA                                 = 0xB000,
      KF_HDR                              = 0xB00A,
      KF_SEG                              = 0xB008,
      KF_CURTIME                          = 0xB009,
      AMBIENT_NODE_TAG                    = 0xB001,
         NODE_ID                          = 0xB030,
         NODE_HDR                         = 0xB010,
         COL_TRACK_TAG                    = 0xB025,
      OBJECT_NODE_TAG                     = 0xB002,
//       NODE_ID
//       NODE_HDR
         PIVOT                            = 0xB013,
         INSTANCE_NAME                    = 0xB011,
         BOUNDBOX                         = 0xB014,
         POS_TRACK_TAG                    = 0xB020,
         ROT_TRACK_TAG                    = 0xB021,
         SCL_TRACK_TAG                    = 0xB022,
         MORPH_TRACK_TAG                  = 0xB026,
         HIDE_TRACK_TAG                   = 0xB029,
         MORPH_SMOOTH                     = 0xB015,
      CAMERA_NODE_TAG                     = 0xB003,
//       NODE_ID
//       NODE_HDR
//       POS_TRACK_TAG
         FOV_TRACK_TAG                    = 0xB023,
         ROLL_TRACK_TAG                   = 0xB024,
      TARGET_NODE_TAG                     = 0xB004,
//       NODE_ID
//       NODE_HDR
//       POS_TRACK_TAG
      LIGHT_NODE_TAG                      = 0xB005,
//       NODE_ID
//       NODE_HDR
//       POS_TRACK_TAG
//       COL_TRACK_TAG
      SPOTLIGHT_NODE_TAG                  = 0xB007,
//       NODE_ID
//       NODE_HDR
//       POS_TRACK_TAG
//       COL_TRACK_TAG
         HOT_TRACK_TAG                    = 0xB027,
         FALL_TRACK_TAG                   = 0xB028,
//       ROLL_TRACK_TAG
      L_TARGET_NODE_TAG                   = 0xB006,
//       NODE_ID
//       NODE_HDR
//       POS_TRACK_TAG

// STRANGE .. NOT USED ..
   FLOAT_PERCENTAGE                       = 0x0031
};



enum {
   CHUNK_NULL        = 0x0000,
   CHUNK_RGBF        = 0x0010,
   CHUNK_RGBB        = 0x0011,
   //CHUNK_RBGB2     = 0x0012,       // ?? NOT HLS.

   CHUNK_PRJ       = 0xC23D,
   CHUNK_MLI       = 0x3DAA,

   CHUNK_MAIN      = 0x4D4D,
      CHUNK_OBJMESH   = 0x3D3D,
         CHUNK_BKGCOLOR  = 0x1200,
         CHUNK_AMBCOLOR  = 0x2100,
         CHUNK_OBJBLOCK  = 0x4000,
            CHUNK_TRIMESH   = 0x4100,
               CHUNK_VERTLIST  = 0x4110,
               CHUNK_FACELIST  = 0x4120,
               CHUNK_FACEMAT   = 0x4130,
               CHUNK_MAPLIST   = 0x4140,
               CHUNK_SMOOLIST  = 0x4150,
               CHUNK_TRMATRIX  = 0x4160,
            CHUNK_LIGHT     = 0x4600,
               CHUNK_SPOTLIGHT = 0x4610,
            CHUNK_CAMERA    = 0x4700,
            CHUNK_HIERARCHY = 0x4F00,
      CHUNK_VIEWPORT  = 0x7001,
      CHUNK_MATERIAL  = 0xAFFF,
         CHUNK_MATNAME   = 0xA000,
         CHUNK_AMBIENT   = 0xA010,
         CHUNK_DIFFUSE   = 0xA020,
         CHUNK_SPECULAR  = 0xA030,
         CHUNK_TEXTURE   = 0xA200,
         CHUNK_BUMPMAP   = 0xA230,
         CHUNK_MAPFILE   = 0xA300,
      CHUNK_KEYFRAMER = 0xB000,
         CHUNK_AMBIENTKEY    = 0xB001,
         CHUNK_TRACKINFO = 0xB002,
            CHUNK_TRACKOBJNAME  = 0xB010,
            CHUNK_TRACKPIVOT    = 0xB013,
            CHUNK_TRACKPOS      = 0xB020,
            CHUNK_TRACKROTATE   = 0xB021,
            CHUNK_TRACKSCALE    = 0xB022,
            CHUNK_OBJNUMBER     = 0xB030,
         CHUNK_TRACKCAMERA = 0xB003,
            CHUNK_TRACKFOV  = 0xB023,
            CHUNK_TRACKROLL = 0xB024,
         CHUNK_TRACKCAMTGT = 0xB004,
         CHUNK_TRACKLIGHT  = 0xB005,
         CHUNK_TRACKLIGTGT = 0xB006,
         CHUNK_TRACKSPOTL  = 0xB007,
         CHUNK_FRAMES    = 0xB008,
};


tds_c::chunk_name_t tds_c::ChunkNames[] =
{
   {CHUNK_RGBF,         "RGB float",               rgb_float_reader},
   {CHUNK_RGBB,         "RGB byte",                rgb_byte_reader},

   {CHUNK_PRJ,          "Project",                 NULL},
   {CHUNK_MLI,          "Material Library",        NULL},

   {CHUNK_MAIN,         "Main",                    chunk_reader},
   {CHUNK_OBJMESH,      "Object Mesh",             chunk_reader},
   {CHUNK_BKGCOLOR,     "Background color",        NULL},
   {CHUNK_AMBCOLOR,     "Ambient color",           NULL},
   {CHUNK_OBJBLOCK,     "Object Block",            object_block_reader},
   {CHUNK_TRIMESH,      "Tri-Mesh",                chunk_reader},
   {CHUNK_VERTLIST,     "Vertex list",             vertex_list_reader},
   {CHUNK_FACELIST,     "Face list",               face_list_reader},
   {CHUNK_FACEMAT,      "Face material",           NULL},
   {CHUNK_MAPLIST,      "Mappings list",           NULL},
   {CHUNK_SMOOLIST,     "Smoothings",              NULL},
   {CHUNK_TRMATRIX,     "Matrix",                  NULL},
   {CHUNK_LIGHT,        "Light",                   NULL},
   {CHUNK_SPOTLIGHT,    "Spotlight",               NULL},
   {CHUNK_CAMERA,       "Camera",                  NULL},
   {CHUNK_HIERARCHY,    "Hierarchy",               NULL},

   {CHUNK_VIEWPORT,     "Viewport info",           NULL},
   {CHUNK_MATERIAL,     "Material",                NULL},
   {CHUNK_MATNAME,      "Material name",           NULL},
   {CHUNK_AMBIENT,      "Ambient color",           ambient_reader},
   {CHUNK_DIFFUSE,      "Diffuse color",           diffuse_reader},
   {CHUNK_SPECULAR,     "Specular color",          specular_reader},
   {CHUNK_TEXTURE,      "Texture map",             NULL},
   {CHUNK_BUMPMAP,      "Bump map",                NULL},
   {CHUNK_MAPFILE,      "Map filename",            NULL},

   {CHUNK_KEYFRAMER,    "Keyframer data",          NULL},
   {CHUNK_AMBIENTKEY,   "Ambient key",             NULL},
   {CHUNK_TRACKINFO,    "Track info",              NULL},
   {CHUNK_FRAMES,       "Frames",                  NULL},
   {CHUNK_TRACKOBJNAME, "Track Obj. Name",         NULL},
   {CHUNK_TRACKPIVOT,   "Pivot point",             NULL},
   {CHUNK_TRACKPOS,     "Position keys",           NULL},
   {CHUNK_TRACKROTATE,  "Rotation keys",           NULL},
   {CHUNK_TRACKSCALE,   "Scale keys",              NULL},
   {CHUNK_OBJNUMBER,    "Object number",           object_number_reader},

   {CHUNK_TRACKCAMERA, "Camera track",             NULL},
   {CHUNK_TRACKCAMTGT, "Camera target track",      NULL},
   {CHUNK_TRACKLIGHT,  "Pointlight track",         NULL},
   {CHUNK_TRACKLIGTGT, "Pointlight target track",  NULL},
   {CHUNK_TRACKSPOTL,  "Spotlight track",          NULL},
   {CHUNK_TRACKFOV,    "FOV track",                NULL},
   {CHUNK_TRACKROLL,   "Roll track",               NULL},
};





void tds_c::
ambient_reader (void)
{
   LastChunk = CHUNK_AMBIENT;
   chunk_reader();
}

void tds_c::
diffuse_reader (void)
{
   LastChunk = CHUNK_DIFFUSE;
   chunk_reader();
}

void tds_c::
specular_reader (void)
{
   LastChunk = CHUNK_SPECULAR;
   chunk_reader();
}


void tds_c::
rgb_float_reader (void)
{
   float c[3];
   if (fread(&c, sizeof(c), 1, File) != 1)
      return;

   switch (LastChunk)
   {
      case CHUNK_LIGHT:
         //light_list[num_lights - 1].color.R = c[0];
         //light_list[num_lights - 1].color.G = c[1];
         //light_list[num_lights - 1].color.B = c[2];
         LastChunk = CHUNK_NULL;
         break;

      case CHUNK_AMBIENT:
         //material_list[num_materials - 1].ambient.R = c[0];
         //material_list[num_materials - 1].ambient.G = c[1];
         //material_list[num_materials - 1].ambient.B = c[2];
         LastChunk = CHUNK_NULL;
         break;

      case CHUNK_DIFFUSE:
         //material_list[num_materials - 1].diffuse.R = c[0];
         //material_list[num_materials - 1].diffuse.G = c[1];
         //material_list[num_materials - 1].diffuse.B = c[2];
         LastChunk = CHUNK_NULL;
         break;

      case CHUNK_SPECULAR:
         //material_list[num_materials - 1].specular.R = c[0];
         //material_list[num_materials - 1].specular.G = c[1];
         //material_list[num_materials - 1].specular.B = c[2];
         LastChunk = CHUNK_NULL;
         break;

      default:
         LastChunk = CHUNK_NULL;
         break;
   }

   if(Verbose)
      printf("Red: %f, Green: %f, Blue: %f\n", c[0], c[1], c[2]);
}


void tds_c::
rgb_byte_reader (void)
{
    byte c[3];
    if (fread(&c, sizeof(c), 1, File) != 1)
      return;

   switch (LastChunk)
   {
      case CHUNK_LIGHT:
         //light_list[num_lights - 1].color.R = c[0];
         //light_list[num_lights - 1].color.G = c[1];
         //light_list[num_lights - 1].color.B = c[2];
         LastChunk = CHUNK_NULL;
         break;

      case CHUNK_AMBIENT:
         //material_list[num_materials - 1].ambient.R = c[0];
         //material_list[num_materials - 1].ambient.G = c[1];
         //material_list[num_materials - 1].ambient.B = c[2];
         LastChunk = CHUNK_NULL;
         break;

      case CHUNK_DIFFUSE:
         //material_list[num_materials - 1].diffuse.R = c[0];
         //material_list[num_materials - 1].diffuse.G = c[1];
         //material_list[num_materials - 1].diffuse.B = c[2];
         LastChunk = CHUNK_NULL;
         break;

      case CHUNK_SPECULAR:
         //material_list[num_materials - 1].specular.R = c[0];
         //material_list[num_materials - 1].specular.G = c[1];
         //material_list[num_materials - 1].specular.B = c[2];
         LastChunk = CHUNK_NULL;
         break;

      default:
         LastChunk = CHUNK_NULL;
         break;
   }

   if(Verbose)
      printf("Red: %d, Green: %d, Blue: %d\n", c[0], c[1], c[2]);
}


void tds_c::
asciiz_reader (void)
{
   int c;

   while ( (c = fgetc(File)) != EOF && c != '\0')
   {
      if(Verbose)
         putchar(c);
   }
   if(Verbose)
      printf("\"\n");
}


void tds_c::
object_block_reader (void)
{
   if(Verbose)
      printf("Object name \"");

   asciiz_reader();
   World->add_object();
   chunk_reader();
}

void tds_c::
vertex_list_reader (void)
{
   word  nb_vertex;
   float coord[3];

   if (fread(&nb_vertex, sizeof(nb_vertex), 1, File) != 1)
      return;

   if(Verbose)
      printf("Vertices: %d\n", nb_vertex);

   World->nb_vertice(World->current_object(),nb_vertex);

   while (nb_vertex > 0)
   {
      if (fread(&coord, sizeof(coord), 1, File) != 1)
         return;

      World->add_vertex(World->current_object(),coord[VX], -coord[VZ], -coord[VY]);
 //     World->add_vertex(World->current_object(), coord[VX], coord[VY], coord[VZ]);

      nb_vertex--;
   }
}

void tds_c::
face_list_reader (void)
{
   word nb_face;
   word coord[3];
   word flags;

   if (fread(&nb_face, sizeof(nb_face), 1, File) != 1)
      return;

   if(Verbose)
      printf("Faces: %d\n", nb_face);

   World->nb_face(World->current_object(),nb_face);

   while (nb_face > 0)
   {
      if (fread(&coord, sizeof(coord), 1, File) != 1)
         return;

      if (fread(&flags, sizeof(flags), 1, File) != 1)
         return;

//      if(Verbose)
      if(FALSE)
      {
         printf("A %d, B %d, C %d, 0x%X\n",
                coord[0], coord[1], coord[2], flags);

         printf(" AB %d BC %d CA %d UWrap %d VWrap %d\n",
               (flags & 0x04) != 0, (flags & 0x02) != 0, (flags & 0x01) != 0,
               (flags & 0x08) != 0, (flags & 0x10) != 0);
      }

      World->add_face(World->current_object(),coord[2], coord[1], coord[0], World->rendering());
//      World->add_face(World->current_object(),coord[0], coord[1], coord[2], World->rendering());

      nb_face --;
   }

   chunk_reader();
}

void tds_c::
object_number_reader(void)
{
    word n;

    if (fread(&n, sizeof(n), 1, File) != 1)
      return;

   if(Verbose)
      printf("Object number: %d\n", n);
}



sword tds_c::
find_chunk(word id)
{
    int i;
    for (i = 0; i < sizeof(ChunkNames)/sizeof(ChunkNames[0]); i++)
        if (id == ChunkNames[i].id)
            return i;
    return -1;
}


void tds_c::
chunk_reader(void)
{
   chunck_header_t header;
   sword chunk_index;

   while ((FilePos = ftell(File)) < FileSize)
   {
      if (fread(&header, sizeof(header), 1, File) != 1)
         return;

      if (header.len == 0)
         return;

      chunk_index = find_chunk(header.id);

      if (chunk_index < 0)
      {
         printf("Chunk 0x%4X [Unknown] skipped\n",header.id);
         fseek(File, FilePos+header.len, SEEK_SET);
      }
      else
      {
         printf("Chunk 0x%4X [%s] ",header.id,ChunkNames[chunk_index].name);

         if (ChunkNames[chunk_index].func != NULL)
         {
            printf("Readed\n");
            ChunkNames[chunk_index].func();
         }
         else
         {
            printf("Skipped\n");
            fseek(File, FilePos+header.len, SEEK_SET);
         }
      }

      if (ferror(File))
         break;
   }
}