#include <dos.h>
#include <conio.h>
#include "vesa.h"
#include "dpmi.h"
#include "macro.h"

// Vesa mode modifier

#define VESA_USE_LFB             0x4000
#define VESA_DONT_CLEAR_SCREEN   0x8000

// Bitfields for VESA SuperVGA mode attributes:

#define VESA_MODE_AVAILABLE      0x0001
#define VESA_MODE_OPTIONAL_INFO  0x0002
#define VESA_MODE_BIOS_OUTPUT    0x0004
#define VESA_MODE_COLOR          0x0008
#define VESA_MODE_GRAPHIC        0x0010
// V 2.0
#define VESA_MODE_NOT_VGA_COMPATIBLE  0x0020
#define VESA_MODE_NO_BANK_SWITCHED    0x0040
#define VESA_MODE_LINEAR              0x0080

// Bitfields for VESA SuperVGA window attributes:
#define VESA_WINDOW_EXIST        0x0001
#define VESA_WINDOW_READABLE     0x0002
#define VESA_WINDOW_WRITABLE     0x0004

// Values for VESA SuperVGA memory model type:
#define VESA_IS_TEXT             0x0000
#define VESA_IS_CGA              0x0001
#define VESA_IS_HGC              0x0002
#define VESA_IS_EGA              0x0003
#define VESA_IS_PACKED_PIXEL     0x0004
#define VESA_IS_256_NON_CHAIN4   0x0005
#define VESA_IS_HICOLOR          0x0006 /* 24 Bits */
#define VESA_IS_YIP              0x0007


// Structure definition

typedef struct info_vesa_s
{
   u8    signature[4];        // ('VESA')
   u16   version;             // Vesa version
   u32   OEMname;             // 32 bits pointer on string
   u8    capabilities[4];
   u32   vesa_list;           // pointer to list of supported VESA and OEM video modes terminated with 0FFFFh.
	u16	TotalMemory;

   // VESA 2.0

   u16   OEMversion;
   u32   vendor_name;         // 32 bits pointer on string
   u32   product_name;        // 32 bits pointer on string
   u32   revision;            // 32 bits pointer on string
	u8    reserved[222];
	u8    OEMscratchpad[256];
}
vesa_base_t;

typedef struct vesa_info_s
{
	// mandatory information
   u16  mode_attributes;      // mode attributes
   u8   wina_attributes;      // window A attributes
   u8   winb_attributes;      // window B attributes
   u16  win_granularity;      // window granularity
   u16  win_size;             // window size in KB
   u16  wina_segment;         // window A start segment
   u16  winb_segment;         // window B start segment
   u32  win_func;             // pointer to windor function (EQ to AX=4F05)
   u16  bytes_per_scanline;   // bytes per scan line

   // formerly optional information (now mandatory)

   u16 x_resolution;             // horizontal resolution
   u16 y_resolution;             // vertical resolution
   u8  x_char_size;              // character cell width
   u8  y_char_size;              // character cell height
   u8  number_of_planes;         // number of memory planes
   u8  bits_per_pixel;           // bits per pixel
   u8  number_of_banks;          // number of banks
   u8  memory_model;             // memory model type
   u8  bank_size;                // bank size in kb
   u8  number_of_image_pages;    // number of images
   u8  reserved;                 // reserved for page function
	// new Direct Color fields (VESA 1.2)
   u8  red_mask_size;            // size of direct color red mask in bits
   u8  red_field_position;       // bit position of LSB of red mask
   u8  green_mask_size;          // size of direct color green mask in bits
   u8  green_field_position;     // bit position of LSB of green mask
   u8  blue_mask_size;           // size of direct color blue mask in bits
   u8  blue_field_position;      // bit position of LSB of blue mask
   u8  reserved_mask_size;       // size of direct color reserved mask in bits
   u8  reserved_field_size;      // bit position of LSB of reserved mask
   u8  direct_color_mode_info;   // Direct Color mode attributes
	// VESA 2.0
   u32 linear_video_buffer;      // Physical address of linear video buffer
   u32 start_offset_screen;      // Pointer to start of offscreen memory
   u16 offset_screen_size;       // Offscreen memory size in KB
   u8  reserved_data[216];       // remainder of ModeInfoBlock
}
vesa_info_t;

// Public Global Variable

vmode_t  Vesa_Mode_Video;
uword    Vesa_Screen_X;
uword    Vesa_Screen_Y;
ubyte    Vesa_Bpp;
ubyte*   Vesa_Start;
ubyte*   Vesa_Video[1024];
bool     Vesa_DAC8bit;
ubyte    Vesa_RM;
ubyte    Vesa_RP;
ubyte    Vesa_GM;
ubyte    Vesa_GP;
ubyte    Vesa_BM;
ubyte    Vesa_BP;

// Private Global structure

vesa_base_t   Vesa_Base;            // Vesa informations
vesa_info_t   Vesa_Info;

// Public in .h

// Private

void vesa_init_system_var (void);
bool vesa_is_mode (vmode_t mode_video);
bool vesa_get_infos (vmode_t video_mode);
bool vesa_set_mode (vmode_t mode_video);
bool vesa_get_linear_adress (udword physaddr);

bool vesa_get_dac_CPaletteontrol (byte* bit_per_color);
bool vesa_set_dac_CPaletteontrol (byte bit_per_color);

bool get_pixel_logical_line_size (word* pixel_per_line_size);
bool set_pixel_logical_line_size (word pixel_per_line_size);
bool vesa_get_byte_logical_line_size (word* byte_per_line_size);
bool vesa_set_byte_logical_line_size (word byte_per_line_size);

void vesa_print_base(void);
void vesa_print_info (void);
void vesa_print_system (void);


//
// Private
//

void inline_wait_vbl (void);
#pragma aux inline_wait_vbl =\
"        mov   edx,3dah " \
"wait1:  in    al,dx    " \
"        test  al,8     " \
"        jz    wait1    " \
"wait2:  in    al,dx    " \
"        test  al,8     " \
"        jnz   wait2    " \
modify [eax edx];

void inline_wait_start_vbl (void);
#pragma aux inline_wait_start_vbl=\
"        mov   edx,3dah " \
"wait1:  in    al,dx    " \
"        test  al,8     " \
"        jz    wait1    " \
modify [eax edx];


void inline_wait_hbl (void);
#pragma aux inline_wait_hbl=\
"        mov   edx,3dah " \
"wait1:  in    al,dx    " \
"        test  al,1     " \
"        jz    wait1    " \
modify [eax edx];


bool vesa_is_mode (vmode_t mode_video)
{
   uword       position  = 0;
   u16 far*    mode_list;

   if (!vesa_is_present())
      return FALSE;

   mode_list = (u16 far*)(RmToPmPtr(Vesa_Base.vesa_list));

   while (mode_list[position] != 0xFFFF && mode_list[position] != mode_video)
      position++;

   return (mode_list[position] == mode_video);
}

bool vesa_get_infos (vmode_t video_mode)
{
   dpmi_regs_t regs;

   memset(&regs,0,sizeof(regs));
   regs.rword.ax = 0x4F01;
   regs.rword.cx = (u16)video_mode;
   dpmi_int_esdi(0x10, &regs,&Vesa_Info,sizeof(vesa_info_t));

   if (Vesa_Info.bits_per_pixel == 8)
      vesa_set_8bit_palette();

   return (regs.rword.ax == 0x004F);
}

bool vesa_set_mode (vmode_t mode_video)
{
   dpmi_regs_t regs;

   memset(&regs,0,sizeof(regs));
   regs.rword.ax = 0x4F02;
	regs.rword.bx = mode_video;
   dpmi_int(0x10, &regs);

   // A lot video card (like Matrox Millenium or Cirrus Logic)
   // set a wrong scanline length .
   if (regs.rword.ax == 0x004F)
   {
      set_pixel_logical_line_size(Vesa_Info.x_resolution);
      return TRUE;
   }
   else
      return FALSE;
}

bool vesa_get_linear_adress (udword physaddr)
{
   udword   limit = (4096L*1024L)-1L;
   dpmi_meminfo_t info;

   selector_t  selector;

   if(dpmi_allocate_ldt_descriptors(1,&selector))
      return FALSE;

   if (dpmi_set_descriptor_access_rights(selector,0x8092))
      return FALSE;

   info.address = physaddr;
   info.size    = limit;

   if (dpmi_physical_address_mapping(&info))
      return FALSE;

   if (dpmi_set_segment_base_address(selector,info.handle))
      return FALSE;
   if (dpmi_set_segment_limit(selector,limit))
      return FALSE;

   if (dpmi_get_segment_base_address(selector,(udword*)&Vesa_Start))
      return FALSE;

   return TRUE;
}


//
// Public
//


bool vesa_is_present (void)
{
   dpmi_regs_t regs;

   memset(&regs,0,sizeof(regs));
   regs.rword.ax = 0x4F00;
   dpmi_int_esdi(0x10,&regs,&Vesa_Base,sizeof(vesa_base_t));

   if(regs.rword.ax == 0x004F)
      return TRUE;
   else
      return FALSE;
}

bool vesa_init_system (vmode_t mode)
{
   uword  index;

   if (!vesa_is_present())                return FALSE;
   if (!vesa_get_infos(mode))             return FALSE;
   if (!vesa_set_mode(mode|VESA_USE_LFB)) return FALSE;

   if (vesa_get_linear_adress(Vesa_Info.linear_video_buffer))
   {
      Vesa_Mode_Video   = mode;
      Vesa_Screen_X     = Vesa_Info.x_resolution;
      Vesa_Screen_Y     = Vesa_Info.y_resolution;
      Vesa_Bpp          = Vesa_Info.bits_per_pixel;
      Vesa_DAC8bit      = 0;
      // Vesa_Start set in vesa_get_linear_adress
      Vesa_RM           = 8-Vesa_Info.red_mask_size;
      Vesa_RP           =   Vesa_Info.red_field_position;
      Vesa_GM           = 8-Vesa_Info.green_mask_size;
      Vesa_GP           =   Vesa_Info.green_field_position;
      Vesa_BM           = 8-Vesa_Info.blue_mask_size;
      Vesa_BP           =   Vesa_Info.blue_field_position;


      // Le +1 & 1111 1110 b est la pour que les mode 15 bits soit alignees
      // comme les mode 16 bits

      for(index=0; index<Vesa_Screen_Y;index++)
         Vesa_Video[index] = Vesa_Start + index*Vesa_Screen_X*(((Vesa_Bpp+1)&0xFE)/8);

      return TRUE;
   }
   else
   {
      // To avoid the crash due to bad programmers

      Vesa_Mode_Video   = 0;
      Vesa_Screen_X     = 1;
      Vesa_Screen_Y     = 1024;
      Vesa_Bpp          = 8;
      Vesa_DAC8bit      = FALSE;
      Vesa_Start        = (ubyte*)malloc(1024+1024+10);
      Vesa_RM           = 0;
      Vesa_RP           = 0;
      Vesa_GM           = 0;
      Vesa_GP           = 0;
      Vesa_BM           = 0;
      Vesa_BP           = 0;

      for(index=0; index<Vesa_Screen_Y;index++)
         Vesa_Video[index] = Vesa_Start + index*Vesa_Screen_X;

      return FALSE;
   }
}

vmode_t vesa_get_mode_by_infos (word width,word height,byte bbp)
{
   vmode_t mode;

   if(vesa_is_present())
      for (mode=0x100; mode<= 0x200; mode++)
      {
         if (vesa_get_infos(mode))
         {
            if
            (
               Vesa_Info.x_resolution == width &&
               Vesa_Info.y_resolution == height &&
               Vesa_Info.bits_per_pixel == bbp
            )
               return mode;
         }
      }

   return 0;
}

bool vesa_init_system (word width,word height,byte bbp)
{
   word mode_find;

   mode_find = vesa_get_mode_by_infos (width,height,bbp);
   if (mode_find)
      return vesa_init_system(mode_find);
   else
      return FALSE;
}

void vesa_original_mode (void)
{
   REGS r;

   r.rword.ax = 0x03;
   int386(0x10,&r,&r);
}


bool vesa_get_current_mode (vmode_t* video_mode)
{
	union REGS regs;

	regs.rword.ax = 0x4F03;
	int386(0x10, &regs, &regs);
	*video_mode = regs.rword.bx;

   return (regs.rword.ax == 0x004F);
}

void vesa_clear_screen (void)
{
   // Le +1 & 1111 1110 b est la pour que les mode 15 bits soit alignees
   // comme les mode 16 bits

   // Optimiser en precalculant le resultat
   memset(Vesa_Start,0,Vesa_Screen_X*Vesa_Screen_Y*(((Vesa_Bpp+1)&0xFE)/8));
}

void vesa_clear_screen (ubyte color)
{
   memset(Vesa_Start,color,Vesa_Screen_X*Vesa_Screen_Y*(((Vesa_Bpp+1)&0xFE)/8));
}


bool vesa_set_8bit_palette (void)
{
   byte bit_per_color;
   bit_per_color = 0;
/*
   if (vesa_set_dac_CPaletteontrol (8))
      vesa_get_dac_CPaletteontrol(&bit_per_color);
   return (bit_per_color == TRUE);
*/
   return TRUE;
}


void vesa_set_color(index_t index, pixel_t red, pixel_t green, pixel_t blue)
{
   outp(0x03c8, (char) index);
   outp(0x03c9, (char) red>>2);
   outp(0x03c9, (char) green>>2);
   outp(0x03c9, (char) blue>>2);
}

void vesa_set_color(index_t index, rgb_t colors)
{
   outp(0x03c8, (char) index);
   outp(0x03c9, (char) colors.red>>2);
   outp(0x03c9, (char) colors.green>>2);
   outp(0x03c9, (char) colors.blue>>2);
}


void vesa_set_palette (index_t start, index_t number, rgb_t* colors)
{
   word count;

   outp(0x03c8, start);
   for(count=0; count<number; count++)
      {
         outp(0x03c9, (char) colors->red>>2);
         outp(0x03c9, (char) colors->green>>2);
         outp(0x03c9, (char) colors->blue>>2);
         colors++;
      }
}


void vesa_wait_vbl (void)
{
   inline_wait_vbl ();
}

void vesa_wait_hbl (void)
{
   inline_wait_hbl ();
}


bool vesa_get_display_start (coord_t* x, coord_t* y)
{
	union REGS regs;

   regs.rword.ax = 0x4F07;
   regs.rword.bx = 0x0001;
	int386(0x10,&regs,&regs);
   *x = regs.rword.cx;
   *y = regs.rword.dx;
   return (regs.rword.ax == 0x004F);
}

bool vesa_set_display_start (coord_t x, coord_t y)
{
	union REGS regs;

   regs.rword.ax = 0x4F07;
   regs.rword.bx = 0x0000;
   regs.rword.cx = x;
   regs.rword.dx = y;
	int386(0x10,&regs,&regs);
   return (regs.rword.ax == 0x004F);
}

bool get_pixel_logical_line_size (word* pixel_per_line_size)
{
	union REGS regs;

   regs.rword.ax = 0x4F06;
   regs.rword.bx = 0x0000;
   regs.rword.cx = 0x0001;
	int386(0x10,&regs,&regs);
   *pixel_per_line_size = regs.rword.bx;
   return (regs.rword.ax == 0x004F);
}

bool set_pixel_logical_line_size (word pixel_per_line_size)
{
   bool  ok = TRUE;
   union REGS regs;

   regs.rword.ax = 0x4F06;
   regs.rword.bx = 0x0000;
   regs.rword.cx = pixel_per_line_size;
   regs.rword.dx = 0x0000;
	int386(0x10,&regs,&regs);
   return (regs.rword.ax == 0x004F);
}

bool vesa_get_byte_logical_line_size (word* byte_per_line_size)
{
	union REGS regs;

   regs.rword.ax = 0x4F06;
   regs.rword.bx = 0x0002;
   regs.rword.cx = 0x0001;
   int386(0x10,&regs,&regs);
   *byte_per_line_size = regs.rword.bx;
   return (regs.rword.ax == 0x004F);
}

bool vesa_set_byte_logical_line_size (word byte_per_line_size)
{
   union REGS regs;

   regs.rword.ax = 0x4F06;
   regs.rword.bx = 0x0002;
   regs.rword.cx = byte_per_line_size;
	int386(0x10,&regs,&regs);
   return (regs.rword.ax == 0x004F);
}

bool vesa_get_dac_CPaletteontrol (byte* bit_per_color)
{
	union REGS regs;

   regs.rword.ax = 0x4F08;
   regs.rword.bx = 0x0001;
	int386(0x10,&regs,&regs);
   *bit_per_color = regs.rbyte.bh;
   return (regs.rword.ax == 0x004F);
}

bool vesa_set_dac_CPaletteontrol (byte bit_per_color)
{
	union REGS regs;

   regs.rword.ax = 0x4F08;
   regs.rbyte.bl = 0x00;
   regs.rbyte.bh = bit_per_color;
	int386(0x10,&regs,&regs);
   return (regs.rword.ax == 0x004F);
}


void vesa_putpixel(coord_t x, coord_t y, byte couleur)
{
	union REGS regs;

   regs.rbyte.ah = 0x0C;
   regs.rbyte.al = couleur;
   regs.rword.bx = 1;
   regs.rword.cx = x;
   regs.rword.dx = y;
	int386(0x10, &regs, &regs);
}

void vesa_print_info (void)
{
   u16 far* mode_list;
   uword    position  = 0;

   mode_list = (u16 far*)(RmToPmPtr(Vesa_Base.vesa_list));
   puts("Liste of the possible video mode");
   while (mode_list[position] != 0xFFFF)
	{
      printf("Mode [%4d] (0x%04X)\n",mode_list[position],mode_list[position]);
      position++;
   }
}

void vesa_print_base(void)
{
   printf ("signature         %c%c%c%c \n"  ,Vesa_Base.signature[0],
                                             Vesa_Base.signature[1],
                                             Vesa_Base.signature[2],
                                             Vesa_Base.signature[3]);
   printf ("OEMname           %s \n"        ,RmToPmPtr(Vesa_Base.OEMname));
   printf ("Vesa list pointer %lu \n"       ,Vesa_Base.vesa_list);
   printf ("\n");
}