#include "camera.h"
#include "fill.h"
#include "light.h"
#include "object.h"
#include "tds.h"
#include "world.h"

#if DEBUG_LEVEL > 0
#  include <conio.h>
#endif

face_filling_function_t RenderingFunction;

void filling (face_c& face)
{
   RenderingFunction(face);
}


void face_sort(ptr_face_c* liste_face, sdword left, sdword right)
{
   vector_t pivot;
   sdword  lower, upper, middle;
   ptr_face_c temp;

   lower  = left;
   upper  = right;
   middle = (left + right) / 2;
   pivot  = liste_face[middle]->center()[VZ];

   do
   {
      while (liste_face[lower]->center()[VZ] < pivot) lower++;
      while (liste_face[upper]->center()[VZ] > pivot) upper--;

      if (lower <= upper)
      {
         temp = liste_face[lower];
         liste_face[lower] = liste_face[upper];
         liste_face[upper] = temp;

         lower++;
         upper--;
      }
   }
   while (lower <= upper);

   if (left < upper)
      face_sort(liste_face,left,upper);

   if (lower < right)
      face_sort(liste_face,lower,right);
}





world_c::
world_c (void)
{
   CurrentObject = -1;
   CurrentMaterial = 0;
   RenderingFunction = fill_flat;
   Camera = new camera_c;
   Light = new light_c;
   Material[0] = new material_c;
}

world_c::
world_c (char* name)
{
   CurrentObject = -1;
   CurrentMaterial = 0;
   RenderingFunction = fill_flat;
   Camera = new camera_c;
   Light = new light_c;
   Material[0] = new material_c;

   initialize_3ds(name);
}

world_c::
~world_c (void)
{
   word idx_obj;
   word idx_mat;

   delete Camera;
   delete Light;
   for(idx_mat=0; idx_mat<CurrentMaterial; idx_mat++)
      delete Material[idx_mat];
   delete ZSort;
//   for(idx_obj=0; idx_obj<CurrentObject;idx_obj++)
//      delete Object[idx_obj];
}

bool world_c::
is_valid (void)
{
   return CurrentObject != -1;
}

void world_c::
initialize_3ds (char* file_name)
{
   printf("Reading 3DS File ");
   tds_c init_3ds(this,file_name);
   printf("Ok.\n Init Engine ");
   init_normale();
   init_material();
   init_zsort();
   printf("Ok.\n");
}

void world_c::
add_object (void)
{
   CurrentObject ++;
   debug_if(CurrentObject>=MAX_OBJECT,"Too many object allocated");
   Object[CurrentObject] = new object_c(this);
}

sword world_c::
current_object (void)
{
   return CurrentObject;
}

void world_c::
nb_vertice (sword obj_index, word number)
{
   Object[obj_index]->nb_vertex(number);
}

void world_c::
add_vertex (sword obj_index, vector_t x, vector_t y, vector_t z)
{
   Object[obj_index]->add_vertex(x,y,z);
}

void world_c::
nb_face (sword obj_index, word number)
{
   Object[obj_index]->nb_face(number);
}

void world_c::
add_face(sword obj_index, word index1, word index2, word index3, face_filling_function_t filling_algo)
{
   Object[obj_index]->add_face(index1,index2,index3,filling_algo);
}


void world_c::
init_normale (void)
{
   word idx_vtx;
   word idx_face;
   word idx_obj;

   for (idx_obj=0; idx_obj<=CurrentObject; idx_obj++)
      for (idx_face=0; idx_face<Object[idx_obj]->nb_face(); idx_face++)
         Object[idx_obj]->Face[idx_face].calcul_normale();

   for (idx_obj=0; idx_obj<=CurrentObject; idx_obj++)
      for (idx_vtx=0; idx_vtx<Object[idx_obj]->nb_vertex(); idx_vtx++)
         Object[idx_obj]->Vertex[idx_vtx].calcul_normale();
}

void world_c::
init_material (void)
{
   word idx_vtx;
   word idx_obj;

   for (idx_obj=0; idx_obj<=CurrentObject; idx_obj++)
      for (idx_vtx=0; idx_vtx<Object[idx_obj]->nb_vertex(); idx_vtx++)
         Object[idx_obj]->Vertex[idx_vtx].calcul_material();
}

void world_c::
init_zsort (void)
{
   word nb_object;
   word nb_face;

   NbFace = 0;

   for (nb_object=0; nb_object<=CurrentObject; nb_object++)
      nb_face += Object[nb_object]->nb_face();

   ZSort = new face_c*[nb_face];
}

void world_c::
rendering (render_t rendering)
{
   switch (rendering)
   {
      case render_flat:
         RenderingFunction = fill_flat;
         break;

      case render_gouraud:
         RenderingFunction = fill_gouraud;
         break;

      case render_phong:
         RenderingFunction = fill_phong;
         break;

      default:
         break;
   }
}

face_filling_function_t world_c::
rendering (void)
{
   return filling;
}


void world_c::
render (void)
{
   word idx_obj;
   word idx_face;
   word idx_vtx;

   for (idx_obj=0; idx_obj<=CurrentObject; idx_obj++)
   {
      for (idx_face=0; idx_face<Object[idx_obj]->nb_face(); idx_face++)
      {
         // rotate Object
         // Clip Object 3D
      }
   }

   // Init Sort

   word ns;
   ns = 0;

   for (idx_obj=0; idx_obj<=CurrentObject; idx_obj++)
   {
      //if(Object[idx_obj]->is_visible())
      {
         for (idx_vtx=0; idx_vtx<Object[idx_obj]->nb_vertex(); idx_vtx++)
         {
            //Object[CurrentObject]->Vertex[idx_vtx].calcul2D(Camera->distance());
            Object[idx_obj]->Vertex[idx_vtx].calcul2D(Camera->distance());
         }

         for (idx_face=0; idx_face<Object[idx_obj]->nb_face(); idx_face++)
         {
            Object[idx_obj]->Face[idx_face].calcul_center();
            //if(visible||twoside)
               ZSort[ns] = &(Object[idx_obj]->Face[idx_face]);
            ns++;
         }
      }
   }

   // Sorting
   face_sort(ZSort,0,ns-1);

   // Rendering Zsorted

   for (idx_face=0; idx_face<ns; idx_face++)
   {
//      printf("Face %d\n",idx_face);
//      flushall();
      ZSort[idx_face]->render(*Light);
   }
}

void world_c::
print_info (void)
{
   word nb_vertex;
   word nb_face;
   word nb_object;

   nb_vertex = 0;
   nb_face   = 0;
   //nb_object = CurrentObject;

   for (nb_object=0; nb_object<=CurrentObject; nb_object++)
   {
      nb_face   += Object[nb_object]->nb_face();
      nb_vertex += Object[nb_object]->nb_vertex();
   }

   printf("Scene Info\n");
   printf("\n");

   printf("Number of Object : %-5d\n",nb_object);
   printf("Number of Face   : %-5d\n",nb_face  );
   printf("Number of Vertex : %-5d\n",nb_vertex);
   printf("\n");
}